# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Modifications made by Cloudera are:
#     Copyright (c) 2016 Cloudera, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
# Passed to session to keep it from finding default config file
import os
import tempfile

from cdpcli import credentials, DEFAULT_PROFILE_NAME
from cdpcli.client import Context
from cdpcli.credentials import EnvProvider
from cdpcli.exceptions import MalformedCredentialsError
from cdpcli.exceptions import NoCredentialsError
from cdpcli.exceptions import PartialCredentialsError
from cdpcli.exceptions import UnknownCredentialError
import mock
from tests import BaseEnvVar
from tests import unittest
from tests.unit import ED25519_KEY


TESTENVVARS = {'config_file': (None, 'CDP_CONFIG_FILE', None)}

raw_metadata = {
    'foobar': {
        'Code': 'Success',
        'LastUpdated': '2012-12-03T14:38:21Z',
        'AccessKeyId': 'foo',
        'PrivateKey': 'bar',
        'Expiration': '2012-12-03T20:48:03Z',
        'Type': 'CDP-HMAC'
    }
}
post_processed_metadata = {
    'role_name': 'foobar',
    'access_key_id': raw_metadata['foobar']['AccessKeyId'],
    'private_key': raw_metadata['foobar']['PrivateKey'],
    'expiry_time': raw_metadata['foobar']['Expiration'],
}
context = Context()


class TestCredentials(BaseEnvVar):
    def _ensure_credential_is_normalized_as_unicode(self, access, secret, token):
        c = credentials.Credentials(access_key_id=access,
                                    private_key=secret,
                                    access_token=token,
                                    method='test')
        self.assertTrue(isinstance(c.access_key_id, type(u'u')))
        self.assertTrue(isinstance(c.private_key, type(u'u')))
        self.assertTrue(isinstance(c.access_token, type(u'u')))

    def test_detect_nonascii_character(self):
        self._ensure_credential_is_normalized_as_unicode(
            'foo\xe2\x80\x99', 'bar\xe2\x80\x99', 'tea\xe2\x80\x99')

    def test_unicode_input(self):
        self._ensure_credential_is_normalized_as_unicode(
            u'foo', u'bar', u'tea')

    def test_frozen_credentials(self):
        cred = credentials.Credentials(access_key_id="key",
                                       private_key="secret",
                                       access_token='token',
                                       method="test")
        frozen_creds = cred.get_frozen_credentials()
        self.assertEqual("key", frozen_creds.access_key_id)
        self.assertEqual("secret", frozen_creds.private_key)
        self.assertEqual("token", frozen_creds.access_token)
        cred.access_key_id = "foobar"
        cred.private_key = "foo"
        cred.access_token = "bar"
        self.assertEqual("key", frozen_creds.access_key_id)
        self.assertEqual("secret", frozen_creds.private_key)
        self.assertEqual("token", frozen_creds.access_token)


class TestBaseCredentialProvider(BaseEnvVar):
    def test_load(self):
        cred_provider = credentials.CredentialProvider()
        self.assertTrue(cred_provider.load())


class TestEnvVar(BaseEnvVar):
    PEM = '----BEGIN MY SECRET---- SECRET ----END MY SECRET----'

    def setUp(self):
        BaseEnvVar.setUp(self)
        fd, self.pem_file = tempfile.mkstemp()
        os.write(fd, TestEnvVar.PEM.encode('utf8'))
        os.close(fd)

    def tearDown(self):
        os.remove(self.pem_file)

    def test_envvars_are_found(self):
        environ = {
            'CDP_ACCESS_KEY_ID': 'foo',
            'CDP_PRIVATE_KEY': self.pem_file,
        }
        provider = credentials.EnvProvider(environ)
        creds = provider.load()
        self.assertIsNotNone(creds)
        self.assertEqual(creds.access_key_id, 'foo')
        self.assertEqual(creds.private_key, TestEnvVar.PEM)
        self.assertEqual(creds.method, 'env')

    def test_envvars_are_found_access_token(self):
        environ = {
            'CDP_ACCESS_TOKEN': 'tea'
        }
        provider = credentials.EnvProvider(environ)
        creds = provider.load()
        self.assertIsNotNone(creds)
        self.assertEqual(creds.access_token, 'tea')
        self.assertEqual(creds.method, 'env')

    def test_envvars_not_found(self):
        provider = credentials.EnvProvider(environ={})
        cred = provider.load()
        self.assertTrue(cred is None)

    def test_can_override_env_var_mapping(self):
        # We can change the env var provider to
        # use our specified env var names.
        environ = {
            'FOO_ACCESS_KEY_id': 'foo',
            'FOO_PRIVATE_KEY': self.pem_file,
        }
        mapping = {
            'access_key_id': 'FOO_ACCESS_KEY_id',
            'private_key': 'FOO_PRIVATE_KEY',
        }
        provider = credentials.EnvProvider(
            environ, mapping
        )
        creds = provider.load()
        self.assertEqual(creds.access_key_id, 'foo')
        self.assertEqual(creds.private_key, TestEnvVar.PEM)

    def test_can_override_env_var_mapping_access_token(self):
        # We can change the env var provider to
        # use our specified env var names.
        environ = {
            'FOO_ACCESS_token': 'foo'
        }
        mapping = {
            'access_token': 'FOO_ACCESS_token'
        }
        provider = credentials.EnvProvider(
            environ, mapping
        )
        creds = provider.load()
        self.assertEqual(creds.access_token, 'foo')

    def test_can_override_partial_env_var_mapping(self):
        # Only changing the access key mapping.
        # The other 2 use the default values of
        # CDP_SECRET_ACCESS_KEY and CDP_SESSION_TOKEN
        # use our specified env var names.
        environ = {
            'FOO_ACCESS_KEY_ID': 'foo',
            'CDP_PRIVATE_KEY': self.pem_file,
        }
        provider = credentials.EnvProvider(
            environ, {'access_key_id': 'FOO_ACCESS_KEY_ID'}
        )
        creds = provider.load()
        self.assertEqual(creds.access_key_id, 'foo')
        self.assertEqual(creds.private_key, TestEnvVar.PEM)

    def test_partial_creds_is_an_error(self):
        # If the user provides an access key, they must also
        # provide a secret key.  Not doing so will generate an
        # error.
        environ = {
            'CDP_ACCESS_KEY_ID': 'foo',
        }
        provider = credentials.EnvProvider(environ)
        with self.assertRaises(PartialCredentialsError):
            provider.load()

    def test_credential_with_envprovider(self):
        # We can change the env var provider to
        # use our specified env var names.
        self.environ['CDP_ACCESS_KEY_ID'] = 'foo'
        self.environ['CDP_PRIVATE_KEY'] = self.pem_file
        creds = credentials.get_credentials(context)
        self.assertEqual(creds.access_key_id, 'foo')
        self.assertEqual(creds.private_key, TestEnvVar.PEM)

    def test_credential_with_envprovider_access_token(self):
        # We can change the env var provider to
        # use our specified env var names.
        self.environ['CDP_ACCESS_TOKEN'] = 'foo'
        creds = credentials.get_credentials(context)
        self.assertEqual(creds.access_token, 'foo')

    def test_raise_if_not_a_file(self):
        environ = {
            'CDP_ACCESS_KEY_ID': 'foo',
            'CDP_PRIVATE_KEY': self.pem_file + "dose_not_exist",
        }
        provider = credentials.EnvProvider(environ)
        with self.assertRaises(NoCredentialsError):
            provider.load()

    def test_ed25519_key_value(self):
        environ = {
            'CDP_ACCESS_KEY_ID': 'foo',
            'CDP_PRIVATE_KEY': ED25519_KEY,
        }
        provider = credentials.EnvProvider(environ)
        creds = provider.load()
        self.assertIsNotNone(creds)
        self.assertEqual(creds.access_key_id, 'foo')
        self.assertEqual(creds.private_key, ED25519_KEY)
        self.assertEqual(creds.method, 'env')


class CredentialResolverTest(BaseEnvVar):
    def setUp(self):
        super(CredentialResolverTest, self).setUp()
        self.provider1 = mock.Mock()
        self.provider1.METHOD = 'provider1'
        self.provider2 = mock.Mock()
        self.provider2.METHOD = 'provider2'
        self.fake_creds = credentials.Credentials(access_key_id='a',
                                                  private_key='b',
                                                  method='test')

    def test_load_credentials_single_provider(self):
        self.provider1.load.return_value = self.fake_creds
        resolver = credentials.CredentialResolver(providers=[self.provider1])
        creds = resolver.load_credentials()
        self.assertEqual(creds.access_key_id, 'a')
        self.assertEqual(creds.private_key, 'b')

    def test_get_provider_by_name(self):
        resolver = credentials.CredentialResolver(providers=[self.provider1])
        result = resolver.get_provider('provider1')
        self.assertIs(result, self.provider1)

    def test_get_unknown_provider_raises_error(self):
        resolver = credentials.CredentialResolver(providers=[self.provider1])
        with self.assertRaises(UnknownCredentialError):
            resolver.get_provider('unknown-foo')

    def test_insert_before_unknown_provider_raises_error(self):
        resolver = credentials.CredentialResolver(providers=[self.provider1])
        with self.assertRaises(UnknownCredentialError):
            resolver.insert_before('unknown-foo', self.provider2)

    def test_first_credential_non_none_wins(self):
        self.provider1.load.return_value = None
        self.provider2.load.return_value = self.fake_creds
        resolver = credentials.CredentialResolver(providers=[self.provider1,
                                                             self.provider2])
        creds = resolver.load_credentials()
        self.assertEqual(creds.access_key_id, 'a')
        self.assertEqual(creds.private_key, 'b')
        self.provider1.load.assert_called_with()
        self.provider2.load.assert_called_with()

    def test_no_creds_loaded(self):
        self.provider1.load.return_value = None
        self.provider2.load.return_value = None
        resolver = credentials.CredentialResolver(providers=[self.provider1,
                                                             self.provider2])
        with self.assertRaises(NoCredentialsError):
            resolver.load_credentials()

    def test_inject_additional_providers_after_existing(self):
        self.provider1.load.return_value = None
        self.provider2.load.return_value = self.fake_creds
        resolver = credentials.CredentialResolver(providers=[self.provider1,
                                                             self.provider2])
        # Now, if we were to call resolver.load() now, provider2 would
        # win because it's returning a non None response.
        # However we can inject a new provider before provider2 to
        # override this process.
        # Providers can be added by the METHOD name of each provider.
        new_provider = mock.Mock()
        new_provider.METHOD = 'new_provider'
        new_provider.load.return_value = credentials.Credentials(access_key_id='d',
                                                                 private_key='e',
                                                                 method='test')

        resolver.insert_after('provider1', new_provider)

        creds = resolver.load_credentials()
        self.assertIsNotNone(creds)

        self.assertEqual(creds.access_key_id, 'd')
        self.assertEqual(creds.private_key, 'e')
        # Provider 1 should have been called, but provider2 should
        # *not* have been called because new_provider already returned
        # a non-None response.
        self.provider1.load.assert_called_with()
        self.assertTrue(not self.provider2.called)

    def test_inject_provider_before_existing(self):
        new_provider = mock.Mock()
        new_provider.METHOD = 'override'
        new_provider.load.return_value = credentials.Credentials(access_key_id='x',
                                                                 private_key='y',
                                                                 method='test')

        resolver = credentials.CredentialResolver(providers=[self.provider1,
                                                             self.provider2])
        resolver.insert_before(self.provider1.METHOD, new_provider)
        creds = resolver.load_credentials()
        self.assertEqual(creds.access_key_id, 'x')
        self.assertEqual(creds.private_key, 'y')

    def test_can_remove_providers(self):
        self.provider1.load.return_value = credentials.Credentials(access_key_id='a',
                                                                   private_key='b',
                                                                   method='test')
        self.provider2.load.return_value = credentials.Credentials(access_key_id='d',
                                                                   private_key='e',
                                                                   method='test')
        resolver = credentials.CredentialResolver(providers=[self.provider1,
                                                             self.provider2])
        resolver.remove('provider1')
        creds = resolver.load_credentials()
        self.assertIsNotNone(creds)
        self.assertEqual(creds.access_key_id, 'd')
        self.assertEqual(creds.private_key, 'e')
        self.assertTrue(not self.provider1.load.called)
        self.provider2.load.assert_called_with()

    def test_provider_unknown(self):
        resolver = credentials.CredentialResolver(providers=[self.provider1,
                                                             self.provider2])
        # No error is raised if you try to remove an unknown provider.
        resolver.remove('providerFOO')
        # But an error IS raised if you try to insert after an unknown
        # provider.
        with self.assertRaises(UnknownCredentialError):
            resolver.insert_after('providerFoo', None)

    def test_explicit_profile_ignores_env_provider(self):
        context = Context('explicit')
        resolver = credentials.create_credential_resolver(context)
        self.assertTrue(
            all(not isinstance(p, EnvProvider) for p in resolver.providers))


class TestCreateCredentialResolver(BaseEnvVar):
    def setUp(self):
        super(TestCreateCredentialResolver, self).setUp()
        self.fake_env_vars = {}

    def test_create_credential_resolver(self):
        resolver = credentials.create_credential_resolver(context)
        self.assertIsInstance(resolver, credentials.CredentialResolver)

    def test_no_profile_checks_env_provider(self):
        resolver = credentials.create_credential_resolver(context)
        # Then an EnvProvider should be part of our credential lookup chain.
        self.assertTrue(
            any(isinstance(p, EnvProvider) for p in resolver.providers))


def _run_test(content, method):
    path = None
    fd = None
    try:
        fd, path = tempfile.mkstemp()
        os.write(fd, content.encode('utf-8'))
        os.close(fd)
        fd = None
        method(path)
        path = None
    finally:
        if fd is not None:
            os.close(fd)
        if path is not None:
            os.remove(path)


class TestAuthConfigFile(unittest.TestCase):
    CONF = '{"access_key_id": "key_id", "private_key": "secret"}'

    def setUp(self):
        fd, self.conf_file = tempfile.mkstemp()
        os.write(fd, TestAuthConfigFile.CONF.encode('utf8'))
        os.close(fd)

    def tearDown(self):
        os.remove(self.conf_file)

    def test_config_is_found(self):
        provider = credentials.AuthConfigFile(self.conf_file)
        creds = provider.load()
        self.assertIsNotNone(creds)
        self.assertEqual(creds.access_key_id, 'key_id')
        self.assertEqual(creds.private_key, 'secret')
        self.assertEqual(creds.method, 'auth_config_file')

    def test_config_is_found_access_token(self):
        def validate(path):
            provider = credentials.AuthConfigFile(path)
            cred = provider.load()
            self.assertIsNotNone(cred)
            self.assertEqual(cred.access_token, 'Bearer A.B.C')
            self.assertEqual(cred.method, 'auth_config_file')

        conf = '{"access_token": "Bearer A.B.C"}'
        _run_test(conf, validate)

    def test_config_not_found(self):
        with self.assertRaises(NoCredentialsError):
            provider = credentials.AuthConfigFile(
                self.conf_file + "_does_not_exist")
            provider.load()

    def test_bad_json(self):
        def validate(path):
            provider = credentials.AuthConfigFile(path)
            cred = provider.load()
            self.assertTrue(cred is None)

        conf = "{'access_key_id': 'key_id'}"
        _run_test(conf, validate)

    def test_partial_creds_is_an_error(self):
        def validate(path):
            with self.assertRaises(PartialCredentialsError):
                provider = credentials.AuthConfigFile(path)
                provider.load()

        conf = '{"access_key_id": "key_id"}'
        _run_test(conf, validate)

    def test_malformed_credentials_in_json(self):
        def validate(path):
            with self.assertRaises(MalformedCredentialsError):
                provider = credentials.AuthConfigFile(path)
                provider.load()

        conf = '{"some_key": "key_id"}'
        _run_test(conf, validate)

    def test_config_is_none(self):
        provider = credentials.AuthConfigFile(None)
        cred = provider.load()
        self.assertTrue(cred is None)


class TestSharedCredentialsProvider(unittest.TestCase):
    CONF = """
[default]
cdp_private_key = secret
cdp_access_key_id = the_key
"""

    def setUp(self):
        fd, self.conf_file = tempfile.mkstemp()
        os.write(fd, TestSharedCredentialsProvider.CONF.encode('utf8'))
        os.close(fd)

    def tearDown(self):
        os.remove(self.conf_file)

    def test_credential_file_exists_default_profile(self):
        provider = credentials.SharedCredentialProvider(
            creds_filename=self.conf_file, profile_name=DEFAULT_PROFILE_NAME)
        creds = provider.load()
        self.assertIsNotNone(creds)
        self.assertEqual(creds.access_key_id, 'the_key')
        self.assertEqual(creds.private_key, 'secret')
        self.assertEqual(creds.method, 'shared-credentials-file')

    def test_partial_creds_missing_private_key_raise_error(self):
        def validate(path):
            provider = credentials.SharedCredentialProvider(
                creds_filename=path, profile_name=DEFAULT_PROFILE_NAME)
            with self.assertRaises(PartialCredentialsError):
                provider.load()

        conf = """
[default]
cdp_access_key_id = the_key
"""
        _run_test(conf, validate)

    def test_partial_creds_missing_key_id_raise_error(self):
        def validate(path):
            provider = credentials.SharedCredentialProvider(
                creds_filename=path, profile_name=DEFAULT_PROFILE_NAME)
            with self.assertRaises(PartialCredentialsError):
                provider.load()

        conf = """
[default]
cdp_private_key = secret
"""
        _run_test(conf, validate)

    def test_credentials_file_with_multiple_profiles(self):
        def validate(path):
            provider = credentials.SharedCredentialProvider(
                creds_filename=path, profile_name='dev')
            creds = provider.load()
            self.assertIsNotNone(creds)
            self.assertEqual(creds.access_key_id, 'admin_key')
            self.assertEqual(creds.private_key, 'admin')
            self.assertEqual(creds.method, 'shared-credentials-file')

        conf = """
[default]
cdp_private_key = secret
cdp_access_key_id = the_key

[dev]
cdp_private_key = admin
cdp_access_key_id = admin_key
"""
        _run_test(conf, validate)

    def test_credentials_file_does_not_exist_returns_none(self):
        # It's ok if the credentials file does not exist, we should
        # just catch the appropriate errors and return None.
        provider = credentials.SharedCredentialProvider(
            creds_filename='/does-not-exist', profile_name='dev')
        creds = provider.load()
        self.assertIsNone(creds)

    def test_newlines_handling(self):
        def validate(path):
            provider = credentials.SharedCredentialProvider(
                creds_filename=path, profile_name=DEFAULT_PROFILE_NAME)
            creds = provider.load()
            self.assertIsNotNone(creds)
            self.assertEqual(creds.access_key_id, 'the_key')
            self.assertEqual(creds.private_key,
                             'secret\nwith\na\nfew\nnewlines')
            self.assertEqual(creds.method, 'shared-credentials-file')

        conf = """
[default]
cdp_private_key = secret\\nwith\\na\\nfew\\nnewlines
cdp_access_key_id = the_key
"""
        _run_test(conf, validate)

    def test_credentials_file_access_token(self):
        def validate(path):
            provider = credentials.SharedCredentialProvider(
                creds_filename=path, profile_name=DEFAULT_PROFILE_NAME)
            creds = provider.load()
            self.assertIsNotNone(creds)
            self.assertEqual(creds.access_token, 'Bearer A.B.C')
            self.assertEqual(creds.method, 'shared-credentials-file')

        conf = """
[default]
cdp_access_token = Bearer A.B.C
"""
        _run_test(conf, validate)


class TestParamsProvider(unittest.TestCase):
    def test_no_param(self):
        provider = credentials.ParamsProvider(None)
        creds = provider.load()
        self.assertIsNone(creds)

    def test_no_access_token_param(self):
        params = {}
        provider = credentials.ParamsProvider(params)
        creds = provider.load()
        self.assertIsNone(creds)

    def test_empty_access_token_param(self):
        params = mock.Mock()
        params.access_token = ''
        provider = credentials.ParamsProvider(params)
        creds = provider.load()
        self.assertIsNone(creds)

    def test_access_token_param(self):
        access_token = 'Bearer A.B'
        params = mock.Mock()
        params.access_token = access_token
        provider = credentials.ParamsProvider(params)
        creds = provider.load()
        self.assertIsNotNone(creds)
        self.assertEqual(creds.access_token, access_token)
        self.assertEqual(creds.method, 'params')



# Copyright (c) 2021 Cloudera, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import os

from cdpcli.exceptions import ClientError, DfExtensionError
from cdpcli.extensions.df.createdeployment import CreateDeploymentOperationCaller
from cdpcli.extensions.df.createdeployment import CreateDeploymentOperationModel
from cdpcli.model import ServiceModel
from mock import Mock
from tests import unittest


BASE_DIR = os.path.normpath(os.path.dirname(os.path.abspath(__file__)) + '/..')


class TestCreateDeployment(unittest.TestCase):

    def setUp(self):
        self.df_client = Mock()
        self.df_client.raise_error.side_effect = Exception('ClientError')

        self.df_workload_client = Mock()
        self.df_workload_client.raise_error.side_effect = Exception('ClientError')

        self.iam_client = Mock()
        self.iam_client.raise_error.side_effect = Exception('ClientError')

        self.client_creator = Mock()

        def _create_client(*args, **kwargs):
            service_name = args[0]
            if service_name == 'df':
                return self.df_client
            elif service_name == 'dfworkload':
                return self.df_workload_client
            elif service_name == 'iam':
                return self.iam_client
            else:
                raise Exception('Unexpected service_name [' + service_name + ']')
        self.client_creator.side_effect = _create_client

        service_model = ServiceModel({}, 'df')
        self.deployment_model = CreateDeploymentOperationModel(service_model)
        self.deployment_caller = CreateDeploymentOperationCaller()

    def test_invoke_create_deployment_environment_crn_not_found(self):
        service_crn = 'SERVICE_CRN'
        flow_version_crn = 'FLOW_VERSION_CRN'
        deployment_name = 'DEPLOYMENT'

        parameters = {
            'serviceCrn': service_crn,
            'flowVersionCrn': flow_version_crn,
            'deploymentName': deployment_name
        }
        parsed_args = {}
        parsed_globals = {}

        response = {
            'services': []
        }
        self.df_client.make_api_call.return_value = (Mock(status_code=200), response)

        with self.assertRaises(DfExtensionError):
            self.deployment_caller.invoke(self.client_creator, self.deployment_model,
                                          parameters, parsed_args, parsed_globals)

    def test_invoke_create_deployment_required_parameters(self):
        self.maxDiff = 2000
        service_crn = 'SERVICE_CRN'
        flow_version_crn = 'FLOW_VERSION_CRN'
        deployment_name = 'DEPLOYMENT'

        parameters = {
            'serviceCrn': service_crn,
            'flowVersionCrn': flow_version_crn,
            'deploymentName': deployment_name
        }

        parsed_args = {}
        parsed_globals = Mock()
        parsed_globals.output = 'json'

        environment_crn = 'ENVIRONMENT_CRN'
        deployment_request_crn = 'DEPLOYMENT_REQUEST_CRN'
        workload_url = 'https://localhost.localdomain/'

        initiate_request_parameters = []

        def _df_make_api_call(*args, **kwargs):
            operation_name = args[0]
            if operation_name == 'listDeployableServicesForNewDeployments':
                return self._get_deployable_services(service_crn, environment_crn)
            elif operation_name == 'initiateDeployment':
                initiate_deployment_response = {
                    'deploymentRequestCrn': deployment_request_crn,
                    'dfxLocalUrl': workload_url
                }
                initiate_request_parameters.append(args[1])
                return (Mock(status_code=200), initiate_deployment_response)
            else:
                raise Exception('Unexpected make_api_call [' + operation_name + ']')
        self.df_client.make_api_call.side_effect = _df_make_api_call

        token = 'WORKLOAD_TOKEN'
        auth_token_response = {
            'token': token,
            'endpointUrl': workload_url
        }
        self.iam_client.generate_workload_auth_token.return_value = auth_token_response

        create_deployment_parameters = []
        deployment_crn = 'DEPLOYMENT_CRN'

        def _df_workload_make_api_call(*args, **kwargs):
            operation_name = args[0]
            if operation_name == 'createDeployment':
                create_response = {
                    'deployment': {
                        'crn': deployment_crn
                    }
                }
                create_deployment_parameters.append(args[1])
                return (Mock(status_code=200), create_response)
            else:
                raise Exception('Unexpected make_api_call [' + operation_name + ']')
        self.df_workload_client.make_api_call.side_effect = _df_workload_make_api_call

        self.deployment_caller.invoke(self.client_creator, self.deployment_model,
                                      parameters, parsed_args, parsed_globals)

        self.assertEquals(
            {
                'serviceCrn': service_crn,
                'flowVersionCrn': flow_version_crn
            },
            initiate_request_parameters[0])

        self.assertEquals('Bearer ' + token, parsed_globals.access_token)
        self.assertEquals(workload_url, parsed_globals.endpoint_url)

        self.assertEquals(
            {
                'environmentCrn': environment_crn,
                'deploymentRequestCrn': deployment_request_crn,
                'name': deployment_name,
                'configurationVersion': 0,
                'clusterSizeName': 'EXTRA_SMALL',
                'staticNodeCount': 1
            },
            create_deployment_parameters[0])

    def test_invoke_create_deployment_auto_scaling_auto_start(self):
        self.maxDiff = 2000
        service_crn = 'SERVICE_CRN'
        flow_version_crn = 'FLOW_VERSION_CRN'
        deployment_name = 'DEPLOYMENT'

        auto_scale_min_nodes = 1
        auto_scale_max_nodes = 3
        parameters = {
            'serviceCrn': service_crn,
            'flowVersionCrn': flow_version_crn,
            'deploymentName': deployment_name,
            'autoScalingEnabled': True,
            'autoScaleMinNodes': auto_scale_min_nodes,
            'autoScaleMaxNodes': auto_scale_max_nodes,
            'autoStartFlow': True,
            'parameterGroups': [],
            'kpis': []
        }

        parsed_args = {}
        parsed_globals = Mock()
        parsed_globals.output = 'json'

        environment_crn = 'ENVIRONMENT_CRN'
        deployment_request_crn = 'DEPLOYMENT_REQUEST_CRN'
        workload_url = 'https://localhost.localdomain/'

        initiate_request_parameters = []

        def _df_make_api_call(*args, **kwargs):
            operation_name = args[0]
            if operation_name == 'listDeployableServicesForNewDeployments':
                return self._get_deployable_services(service_crn, environment_crn)
            elif operation_name == 'initiateDeployment':
                initiate_deployment_response = {
                    'deploymentRequestCrn': deployment_request_crn,
                    'dfxLocalUrl': workload_url
                }
                initiate_request_parameters.append(args[1])
                return (Mock(status_code=200), initiate_deployment_response)
            else:
                raise Exception('Unexpected make_api_call [' + operation_name + ']')
        self.df_client.make_api_call.side_effect = _df_make_api_call

        token = 'WORKLOAD_TOKEN'
        auth_token_response = {
            'token': token,
            'endpointUrl': workload_url
        }
        self.iam_client.generate_workload_auth_token.return_value = auth_token_response

        create_deployment_parameters = []
        deployment_crn = 'DEPLOYMENT_CRN'

        def _df_workload_make_api_call(*args, **kwargs):
            operation_name = args[0]
            if operation_name == 'createDeployment':
                create_response = {
                    'deployment': {
                        'crn': deployment_crn
                    }
                }
                create_deployment_parameters.append(args[1])
                return (Mock(status_code=200), create_response)
            else:
                raise Exception('Unexpected make_api_call [' + operation_name + ']')
        self.df_workload_client.make_api_call.side_effect = _df_workload_make_api_call

        self.deployment_caller.invoke(self.client_creator, self.deployment_model,
                                      parameters, parsed_args, parsed_globals)

        self.assertEquals(
            {
                'serviceCrn': service_crn,
                'flowVersionCrn': flow_version_crn
            },
            initiate_request_parameters[0])

        self.assertEquals('Bearer ' + token, parsed_globals.access_token)
        self.assertEquals(workload_url, parsed_globals.endpoint_url)

        self.assertEquals(
            {
                'environmentCrn': environment_crn,
                'deploymentRequestCrn': deployment_request_crn,
                'name': deployment_name,
                'configurationVersion': 0,
                'clusterSizeName': 'EXTRA_SMALL',
                'autoScalingEnabled': True,
                'autoScaleMinNodes': auto_scale_min_nodes,
                'autoScaleMaxNodes': auto_scale_max_nodes,
                'autoStartFlow': True
            },
            create_deployment_parameters[0])

    def test_invoke_create_deployment(self):
        self.maxDiff = 2000
        service_crn = 'SERVICE_CRN'
        flow_version_crn = 'FLOW_VERSION_CRN'
        deployment_name = 'DEPLOYMENT'

        parameter_group_name = 'ParameterGroup'
        parameter_name = 'Files'
        asset_reference_name = 'df-workload.asset.bin'
        asset_reference_file_path = os.path.join(BASE_DIR, asset_reference_name)

        frequencyUnit = {
            'id': 'MINUTES'
        }
        frequencyUnitUpdated = {
            'id': 'MINUTES',
            'label': 'Minutes',
            'abbreviation': 'm'
        }
        kpi = {
            'metricId': 'cpuUtilization',
            'alert': {
                'thresholdMoreThan': {
                    'unitId': 'percentage',
                    'value': 75.5
                },
                'frequencyTolerance': {
                    'unit': frequencyUnit,
                    'value': 2.5
                }
            }
        }

        custom_nar_configuration = {
            'username': 'USER',
            'password': 'PROTECTED',
            'storageLocation': 's3a://bucket',
            'configurationVersion': 0
        }

        cluster_size_name = 'MEDIUM'
        auto_scaling_enabled = False
        static_node_count = 2
        cfm_nifi_version = '1.14.0'
        auto_start_flow = False
        parameters = {
            'serviceCrn': service_crn,
            'flowVersionCrn': flow_version_crn,
            'deploymentName': deployment_name,
            'clusterSizeName': cluster_size_name,
            'autoScalingEnabled': auto_scaling_enabled,
            'staticNodeCount': static_node_count,
            'cfmNifiVersion': cfm_nifi_version,
            'autoStartFlow': auto_start_flow,
            'parameterGroups': [
                {
                    'name': parameter_group_name,
                    'parameters': [
                        {
                            'name': parameter_name,
                            'assetReferences': [
                                asset_reference_file_path
                            ]
                        }
                    ]
                }
            ],
            'kpis': [
                kpi
            ],
            'customNarConfiguration': custom_nar_configuration
        }
        parsed_args = {}
        parsed_globals = Mock()
        parsed_globals.output = 'json'

        environment_crn = 'ENVIRONMENT_CRN'
        deployment_request_crn = 'DEPLOYMENT_REQUEST_CRN'
        custom_nar_configuration_crn = 'NAR_CONFIGURATION_CRN'
        workload_url = 'https://localhost.localdomain/'

        initiate_request_parameters = []

        def _df_make_api_call(*args, **kwargs):
            operation_name = args[0]
            if operation_name == 'listDeployableServicesForNewDeployments':
                return self._get_deployable_services(service_crn, environment_crn)
            elif operation_name == 'initiateDeployment':
                initiate_deployment_response = {
                    'deploymentRequestCrn': deployment_request_crn,
                    'dfxLocalUrl': workload_url
                }
                initiate_request_parameters.append(args[1])
                return (Mock(status_code=200), initiate_deployment_response)
            else:
                raise Exception('Unexpected make_api_call [' + operation_name + ']')
        self.df_client.make_api_call.side_effect = _df_make_api_call

        token = 'WORKLOAD_TOKEN'
        auth_token_response = {
            'token': token,
            'endpointUrl': workload_url
        }
        self.iam_client.generate_workload_auth_token.return_value = auth_token_response

        create_deployment_parameters = []
        deployment_crn = 'DEPLOYMENT_CRN'

        create_custom_nar_configuration_params = []

        def _df_workload_make_api_call(*args, **kwargs):
            operation_name = args[0]
            if operation_name == 'createDeployment':
                create_response = {
                    'deployment': {
                        'crn': deployment_crn
                    }
                }
                create_deployment_parameters.append(args[1])
                return (Mock(status_code=200), create_response)
            elif operation_name == 'getDefaultCustomNarConfiguration':
                error = {
                    'error': {
                        'code': '404',
                        'message': 'Not Found'
                    }
                }
                raise ClientError(error, operation_name, 'dfworkload', 404, 'requestId')
            elif operation_name == 'createCustomNarConfiguration':
                create_custom_nar_configuration_params.append(args[1])
                configuration_response = {
                    'crn': custom_nar_configuration_crn
                }
                return (Mock(status_code=200), configuration_response)
            else:
                raise Exception('Unexpected make_api_call [' + operation_name + ']')
        self.df_workload_client.make_api_call.side_effect = _df_workload_make_api_call

        upload_parameters = []

        def _df_workload_make_request(*args, **kwargs):
            operation_name = args[0]
            if operation_name == 'uploadAsset':
                upload_parameters.append(args[3])
                return (Mock(status_code=200), {})
            else:
                raise Exception('Unexpected make_request [' + operation_name + ']')
        self.df_workload_client.make_request.side_effect = _df_workload_make_request

        self.deployment_caller.invoke(self.client_creator, self.deployment_model,
                                      parameters, parsed_args, parsed_globals)

        self.assertEquals(
            {
                'serviceCrn': service_crn,
                'flowVersionCrn': flow_version_crn
            },
            initiate_request_parameters[0])

        self.assertEquals('Bearer ' + token, parsed_globals.access_token)
        self.assertEquals(workload_url, parsed_globals.endpoint_url)

        self.assertEquals(
            {
                'Content-Type': 'application/octet-stream',
                'Deployment-Name': deployment_name,
                'Deployment-Request-Crn': deployment_request_crn,
                'File-Path': asset_reference_file_path,
                'Parameter-Group': parameter_group_name,
                'Parameter-Name': parameter_name
            },
            upload_parameters[0]
        )

        kpi['alert']['frequencyTolerance']['unit'] = frequencyUnitUpdated
        self.assertEquals(
            {
                'environmentCrn': environment_crn,
                'deploymentRequestCrn': deployment_request_crn,
                'name': deployment_name,
                'configurationVersion': 0,
                'clusterSizeName': cluster_size_name,
                'staticNodeCount': static_node_count,
                'cfmNifiVersion': cfm_nifi_version,
                'autoStartFlow': auto_start_flow,
                'autoScalingEnabled': auto_scaling_enabled,
                'parameterGroups': [
                    {
                        'name': parameter_group_name,
                        'parameters': [
                            {
                                'name': parameter_name,
                                'assetReferences': [
                                    {
                                        'name': asset_reference_name,
                                        'path': BASE_DIR,
                                        'version': '0'
                                    }
                                ]
                            }
                        ]
                    }
                ],
                'kpis': [
                    kpi
                ],
                'customNarConfigurationCrn': custom_nar_configuration_crn
            },
            create_deployment_parameters[0])

        custom_nar_configuration['environmentCrn'] = environment_crn
        self.assertEquals(
            custom_nar_configuration,
            create_custom_nar_configuration_params[0]
        )

    def test_invoke_create_deployment_default_custom_nar_configuration_found(self):
        self.maxDiff = 2000
        service_crn = 'SERVICE_CRN'
        flow_version_crn = 'FLOW_VERSION_CRN'
        deployment_name = 'DEPLOYMENT'
        custom_nar_configuration_crn = 'NAR_CONFIGURATION_CRN'

        custom_nar_configuration = {
            'username': 'UPDATED',
            'password': 'PROTECTED',
            'storageLocation': 's3a://bucket',
            'configurationVersion': 1
        }

        default_custom_nar_configuration = {
            'crn': custom_nar_configuration_crn,
            'username': 'USER',
            'password': 'PROTECTED',
            'storageLocation': 's3a://bucket',
            'configurationVersion': 1
        }

        cluster_size_name = 'MEDIUM'
        auto_scaling_enabled = False
        static_node_count = 2
        cfm_nifi_version = '1.14.0'
        auto_start_flow = False
        parameters = {
            'serviceCrn': service_crn,
            'flowVersionCrn': flow_version_crn,
            'deploymentName': deployment_name,
            'clusterSizeName': cluster_size_name,
            'autoScalingEnabled': auto_scaling_enabled,
            'staticNodeCount': static_node_count,
            'cfmNifiVersion': cfm_nifi_version,
            'autoStartFlow': auto_start_flow,
            'customNarConfiguration': custom_nar_configuration
        }
        parsed_args = {}
        parsed_globals = Mock()
        parsed_globals.output = 'json'

        environment_crn = 'ENVIRONMENT_CRN'
        deployment_request_crn = 'DEPLOYMENT_REQUEST_CRN'
        workload_url = 'https://localhost.localdomain/'

        initiate_request_parameters = []

        def _df_make_api_call(*args, **kwargs):
            operation_name = args[0]
            if operation_name == 'listDeployableServicesForNewDeployments':
                return self._get_deployable_services(service_crn, environment_crn)
            elif operation_name == 'initiateDeployment':
                initiate_deployment_response = {
                    'deploymentRequestCrn': deployment_request_crn,
                    'dfxLocalUrl': workload_url
                }
                initiate_request_parameters.append(args[1])
                return (Mock(status_code=200), initiate_deployment_response)
            else:
                raise Exception('Unexpected make_api_call [' + operation_name + ']')
        self.df_client.make_api_call.side_effect = _df_make_api_call

        token = 'WORKLOAD_TOKEN'
        auth_token_response = {
            'token': token,
            'endpointUrl': workload_url
        }
        self.iam_client.generate_workload_auth_token.return_value = auth_token_response

        create_deployment_parameters = []
        deployment_crn = 'DEPLOYMENT_CRN'

        create_custom_nar_configuration_params = []

        def _df_workload_make_api_call(*args, **kwargs):
            operation_name = args[0]
            if operation_name == 'createDeployment':
                create_response = {
                    'deployment': {
                        'crn': deployment_crn
                    }
                }
                create_deployment_parameters.append(args[1])
                return (Mock(status_code=200), create_response)
            elif operation_name == 'getDefaultCustomNarConfiguration':
                return (Mock(status_code=200), default_custom_nar_configuration)
            elif operation_name == 'updateCustomNarConfiguration':
                create_custom_nar_configuration_params.append(args[1])
                configuration_response = {
                    'crn': custom_nar_configuration_crn
                }
                return (Mock(status_code=200), configuration_response)
            else:
                raise Exception('Unexpected make_api_call [' + operation_name + ']')
        self.df_workload_client.make_api_call.side_effect = _df_workload_make_api_call

        upload_parameters = []

        def _df_workload_make_request(*args, **kwargs):
            operation_name = args[0]
            if operation_name == 'uploadAsset':
                upload_parameters.append(args[3])
                return (Mock(status_code=200), {})
            else:
                raise Exception('Unexpected make_request [' + operation_name + ']')
        self.df_workload_client.make_request.side_effect = _df_workload_make_request

        self.deployment_caller.invoke(self.client_creator, self.deployment_model,
                                      parameters, parsed_args, parsed_globals)

        self.assertEquals(
            {
                'serviceCrn': service_crn,
                'flowVersionCrn': flow_version_crn
            },
            initiate_request_parameters[0])

        self.assertEquals('Bearer ' + token, parsed_globals.access_token)
        self.assertEquals(workload_url, parsed_globals.endpoint_url)

        self.assertEquals(
            {
                'environmentCrn': environment_crn,
                'deploymentRequestCrn': deployment_request_crn,
                'name': deployment_name,
                'configurationVersion': 0,
                'clusterSizeName': cluster_size_name,
                'staticNodeCount': static_node_count,
                'cfmNifiVersion': cfm_nifi_version,
                'autoStartFlow': auto_start_flow,
                'autoScalingEnabled': auto_scaling_enabled,
                'customNarConfigurationCrn': custom_nar_configuration_crn
            },
            create_deployment_parameters[0])

        custom_nar_configuration['environmentCrn'] = environment_crn
        self.assertEquals(
            custom_nar_configuration,
            create_custom_nar_configuration_params[0]
        )

    def test_invoke_create_deployment_initiate_error(self):
        self.maxDiff = 2000
        service_crn = 'SERVICE_CRN'
        flow_version_crn = 'FLOW_VERSION_CRN'
        deployment_name = 'DEPLOYMENT'

        parameters = {
            'serviceCrn': service_crn,
            'flowVersionCrn': flow_version_crn,
            'deploymentName': deployment_name
        }

        parsed_args = {}
        parsed_globals = Mock()
        parsed_globals.output = 'json'

        environment_crn = 'ENVIRONMENT_CRN'

        def _df_make_api_call(*args, **kwargs):
            operation_name = args[0]
            if operation_name == 'listDeployableServicesForNewDeployments':
                return self._get_deployable_services(service_crn, environment_crn)
            elif operation_name == 'initiateDeployment':
                error_response = {
                    'error': {
                        'code': '500',
                        'message': 'Server Error'
                    }
                }
                raise ClientError(error_response, operation_name, 'df', 500, 'requestId')
            else:
                raise Exception('Unexpected make_api_call [' + operation_name + ']')
        self.df_client.make_api_call.side_effect = _df_make_api_call

        with self.assertRaises(ClientError):
            self.deployment_caller.invoke(self.client_creator, self.deployment_model,
                                          parameters, parsed_args, parsed_globals)

        self.assertEqual(2, self.df_client.make_api_call.call_count)
        self.assertEqual(0, self.iam_client.make_api_call.call_count)
        self.assertEqual(0, self.df_workload_client.make_api_call.call_count)

    def test_invoke_create_deployment_workload_error(self):
        self.maxDiff = 2000
        service_crn = 'SERVICE_CRN'
        flow_version_crn = 'FLOW_VERSION_CRN'
        deployment_name = 'DEPLOYMENT'

        parameters = {
            'serviceCrn': service_crn,
            'flowVersionCrn': flow_version_crn,
            'deploymentName': deployment_name
        }

        parsed_args = {}
        parsed_globals = Mock()
        parsed_globals.output = 'json'

        environment_crn = 'ENVIRONMENT_CRN'
        deployment_request_crn = 'DEPLOYMENT_REQUEST_CRN'
        workload_url = 'https://localhost.localdomain/'

        initiate_request_parameters = []

        def _df_make_api_call(*args, **kwargs):
            operation_name = args[0]
            if operation_name == 'listDeployableServicesForNewDeployments':
                return self._get_deployable_services(service_crn, environment_crn)
            elif operation_name == 'initiateDeployment':
                initiate_deployment_response = {
                    'deploymentRequestCrn': deployment_request_crn,
                    'dfxLocalUrl': workload_url
                }
                initiate_request_parameters.append(args[1])
                return (Mock(status_code=200), initiate_deployment_response)
            else:
                raise Exception('Unexpected make_api_call [' + operation_name + ']')
        self.df_client.make_api_call.side_effect = _df_make_api_call

        token = 'WORKLOAD_TOKEN'
        auth_token_response = {
            'token': token,
            'endpointUrl': workload_url
        }
        self.iam_client.generate_workload_auth_token.return_value = auth_token_response

        def _df_workload_make_api_call(*args, **kwargs):
            operation_name = args[0]
            if operation_name == 'createDeployment':
                error_response = {
                    'error': {
                        'code': '403',
                        'message': 'Access Denied'
                    }
                }
                raise ClientError(error_response, operation_name, 'df', 403, 'requestId')
            else:
                raise Exception('Unexpected make_api_call [' + operation_name + ']')
        self.df_workload_client.make_api_call.side_effect = _df_workload_make_api_call

        with self.assertRaises(ClientError):
            self.deployment_caller.invoke(self.client_creator, self.deployment_model,
                                          parameters, parsed_args, parsed_globals)

        self.assertEqual(2, self.df_client.make_api_call.call_count)
        self.assertEqual(1, self.iam_client.generate_workload_auth_token.call_count)
        self.assertEqual(1, self.df_workload_client.make_api_call.call_count)

    def _get_deployable_services(self, service_crn, environment_crn):
        response = {
            'services': [
                {
                    'crn': service_crn,
                    'environmentCrn': environment_crn
                }
            ]
        }
        return (Mock(status_code=200), response)



# Copyright (c) 2021 Cloudera, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import os

from cdpcli.exceptions import DfExtensionError
from cdpcli.extensions.df import upload_file
from cdpcli.extensions.df.createdeployment import CreateDeploymentOperationModel
from cdpcli.model import ServiceModel
from mock import Mock
from tests import unittest


BASE_DIR = os.path.normpath(os.path.dirname(os.path.abspath(__file__)) + '/..')


class TestUploadFile(unittest.TestCase):

    def setUp(self):
        self.df_client = Mock()
        self.df_client.raise_error.side_effect = Exception('ClientError')

        service_model = ServiceModel({}, 'df')
        self.deployment_model = CreateDeploymentOperationModel(service_model)

    def test_upload_file_path(self):
        file_name = 'df-workload.asset.bin'
        file_path = os.path.join(BASE_DIR, file_name)
        headers = {
            'Content-Type': 'application/octet-stream'
        }

        expected = {
            'crn': 'UPLOAD_CRN'
        }
        self.df_client.make_request.return_value = (Mock(status_code=200), expected)

        response = upload_file(self.df_client,
                               'importFlowDefinition',
                               'POST', 'http://localhost', headers, file_path)

        self.assertEquals(expected, response)

    def test_upload_file_path_not_found(self):
        file_name_not_found = 'file_not_found'
        file_path = os.path.join(BASE_DIR, file_name_not_found)
        headers = {
            'Content-Type': 'application/octet-stream'
        }

        with self.assertRaises(DfExtensionError):
            upload_file(self.df_client,
                        'importFlowDefinition',
                        'POST', 'http://localhost', headers, file_path)



# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Modifications made by Cloudera are:
#     Copyright (c) 2016 Cloudera, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import cdpcli
import cdpcli.auth
from cdpcli.credentials import Credentials
from cdpcli.exceptions import UnknownSignatureVersionError
from cdpcli.signers import RequestSigner
import mock
from tests import unittest


class BaseSignerTest(unittest.TestCase):
    def setUp(self):
        self.credentials = Credentials(access_key_id='key',
                                       private_key='secret',
                                       method='test')
        self.signer = RequestSigner('rsav1', self.credentials)
        self.fixed_credentials = self.credentials.get_frozen_credentials()


class TestSigner(BaseSignerTest):

    def test_signature_version(self):
        self.assertEqual(self.signer.signature_version, 'rsav1')

    def test_get_auth(self):
        auth_cls = mock.Mock()
        with mock.patch.dict(cdpcli.auth.AUTH_TYPE_MAPS,
                             {'rsav1': auth_cls}):
            auth = self.signer.get_auth_instance('rsav1',
                                                 extra_param='extra_param')

            self.assertEqual(auth, auth_cls.return_value)
            auth_cls.assert_called_with(
                credentials=self.fixed_credentials,
                extra_param='extra_param')

    def test_get_auth_signature_override(self):
        auth_cls = mock.Mock()
        with mock.patch.dict(cdpcli.auth.AUTH_TYPE_MAPS,
                             {'rsav1-custom': auth_cls}):
            auth = self.signer.get_auth_instance('rsav1-custom',
                                                 extra_param='extra_param')

            self.assertEqual(auth, auth_cls.return_value)
            auth_cls.assert_called_with(
                credentials=self.fixed_credentials,
                extra_param='extra_param')

    def test_get_auth_bad_override(self):
        with self.assertRaises(UnknownSignatureVersionError):
            self.signer.get_auth_instance('bad')

    def test_disable_signing(self):
        request = mock.Mock()
        auth = mock.Mock()
        self.signer = RequestSigner(cdpcli.UNSIGNED, self.credentials)
        with mock.patch.dict(cdpcli.auth.AUTH_TYPE_MAPS,
                             {'rsav1': auth}):
            self.signer.sign(request)
        auth.assert_not_called()

    def test_sign(self):
        request = mock.Mock()
        auth_cls = mock.Mock()
        with mock.patch.dict(cdpcli.auth.AUTH_TYPE_MAPS,
                             {'rsav1': auth_cls}):
            self.signer.sign(request)
            auth_cls.assert_called_with(credentials=self.fixed_credentials)



# Copyright 2012-2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Modifications made by Cloudera are:
#     Copyright (c) 2016 Cloudera, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import os

from cdpcli.client import ClientCreator
from cdpcli.exceptions import ClientError
from cdpcli.exceptions import OperationNotPageableError
from cdpcli.exceptions import ParamValidationError
from cdpcli.parser import ResponseParserFactory
from mock import Mock
from tests import unittest
from tests.unit import FakeContext
import yaml


USE_DEFAULT_ENDPOINT_URL = None
DEFAULT_REGION = 'us-west-1'
VERIFY_TLS = True
CLIENT_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'client')


class TestClient(unittest.TestCase):

    def setUp(self):
        self.loader = Mock()
        self.loader.load_service_data.return_value = yaml.safe_load(
            open(os.path.join(CLIENT_DIR, 'service.yaml')))
        self.endpoint = Mock()
        self.endpoint.host = 'http://thunderhead.cloudera.altus.cloudera.com'
        self.endpoint.make_request.return_value = (Mock(status_code=200), {})

        self.endpoint_creator = Mock()
        self.endpoint_creator.create_endpoint.return_value = self.endpoint
        self.retryhandler = Mock()
        self.context = FakeContext({})
        self.client_creator = ClientCreator(self.loader,
                                            self.context,
                                            self.endpoint_creator,
                                            'user-agent-header',
                                            ResponseParserFactory(),
                                            self.retryhandler)
        self.credentials = Mock()
        self.credentials.private_key = "A Private Key"

    def _create_client(self):
        return self.client_creator.create_client(
            'thunderhead', USE_DEFAULT_ENDPOINT_URL, DEFAULT_REGION,
            VERIFY_TLS, self.credentials)

    def _get_request_params(self):
        self.assertTrue(self.endpoint.make_request.called)
        return dict((k.lower(), v) for k, v in
                    self.endpoint.make_request.call_args[0][1].items())

    def test_client_name(self):
        client = self._create_client()
        self.assertEquals(client.__class__.__name__, 'thunderhead')

    def test_client_meta(self):
        client = self._create_client()
        self.assertTrue(client.meta is not None)
        self.assertTrue(client.meta.service_model is not None)
        self.assertEquals(client.meta.endpoint_url,
                          'http://thunderhead.cloudera.altus.cloudera.com')

    def test_client_operations(self):
        client = self._create_client()
        self.assertTrue(hasattr(client, 'create_director'))
        self.assertFalse(hasattr(client, 'non_existent_operation'))

    def test_client_method_to_api_mapping(self):
        client = self._create_client()
        self.assertEqual(client.meta.method_to_api_mapping['create_director'],
                         'createDirector')

    def test_client_call(self):
        client = self._create_client()
        response = client.describe_directors()
        self.assertTrue(self.endpoint.make_request.called)
        self.assertEqual(response, {})

    def test_get_paginator(self):
        client = self._create_client()
        self.assertTrue(client.get_paginator('describe_directors') is not None)
        with self.assertRaises(OperationNotPageableError):
            client.get_paginator('create_director')

    def test_client_user_agent_in_request(self):
        client = self._create_client()
        client.describe_directors()
        self.assertTrue(self.endpoint.make_request.called)
        params = self._get_request_params()
        self.assertEqual(params['headers']['User-Agent'], 'user-agent-header')

    def test_additional_headers_not_in_request_by_default(self):
        client = self._create_client()
        client.describe_directors()
        self.assertTrue(self.endpoint.make_request.called)
        params = self._get_request_params()
        self.assertEquals(2, len(params['headers']))

    def test_additional_headers_from_config(self):
        self.context.config['request_headers'] = \
            'x-test-1=crn:altus:iam,x-test-2=generate::uuid'
        client = self._create_client()
        client.describe_directors()
        self.assertTrue(self.endpoint.make_request.called)
        params = self._get_request_params()
        self.assertEquals('crn:altus:iam', params['headers']['x-test-1'])
        self.assertTrue('x-test-2' in params['headers'])
        self.assertNotEquals('generate::uuid', params['headers']['x-test-2'])

    def test_client_error_message_for_positional_args(self):
        client = self._create_client()
        with self.assertRaisesRegexp(TypeError, 'only accepts keyword arguments'):
            client.create_director('foo')

    def test_client_error(self):
        client = self._create_client()
        error = {'error': {'code': 'test-code', 'message': 'test-message'}}
        mock_response = Mock(
            status_code=400,
            headers={'x-altus-request-id': '00000000-0000-0000-0000-000000000000'})
        self.endpoint.make_request.return_value = (mock_response, error)
        re = (
            'An error occurred: test-message '
            '\\(Status Code: 400; '
            'Error Code: test-code; '
            'Service: thunderhead; '
            'Operation: describeDirectors; '
            'Request ID: 00000000-0000-0000-0000-000000000000;\\)')
        with self.assertRaisesRegexp(ClientError, re) as e:
            client.describe_directors()
        self.assertEqual(e.exception.response['error']['code'], 'test-code')
        self.assertEqual(e.exception.response['error']['message'], 'test-message')

    def test_client_error_cdp_request_id(self):
        client = self._create_client()
        error = {'error': {'code': 'test-code', 'message': 'test-message'}}
        mock_response = Mock(
            status_code=400,
            headers={'x-cdp-request-id': '0000000a-0000-0000-0000-000000000001'})
        self.endpoint.make_request.return_value = (mock_response, error)
        re = 'Request ID: 0000000a-0000-0000-0000-000000000001;'
        with self.assertRaisesRegexp(ClientError, re):
            client.describe_directors()

    def test_client_validates_params(self):
        client = self._create_client()
        with self.assertRaises(ParamValidationError):
            client.create_director()

    def test_operation_cannot_paginate(self):
        client = self._create_client()
        self.assertTrue(hasattr(client, 'create_director'))
        self.assertFalse(client.can_paginate('create_director'))

    def test_operation_can_paginate(self):
        client = self._create_client()
        self.assertTrue(hasattr(client, 'describe_directors'))
        self.assertTrue(client.can_paginate('describe_directors'))

    def test_make_request(self):
        client = self._create_client()
        response = {'foo': 'bar'}
        self.endpoint.make_request.return_value = (Mock(status_code=200), response)
        http, parsed_response = client.make_request('describeDirectors',
                                                    'post',
                                                    '/directors/describeDirectors',
                                                    {'test': 'value'},
                                                    b'body')
        self.assertTrue(self.endpoint.make_request.called)
        params = self._get_request_params()
        self.assertEquals('post', params['method'])
        self.assertEquals('/directors/describeDirectors', params['url_path'])
        self.assertEquals(
            'http://thunderhead.cloudera.altus.cloudera.com/directors/describeDirectors',
            params['url'])
        self.assertEquals('value', params['headers']['test'])
        self.assertEquals(b'body', params['body'])
        kwargs = self.endpoint.make_request.call_args[1]
        self.assertEquals({'allow_redirects': True}, kwargs)
        self.assertEquals(200, http.status_code)
        self.assertEquals(response, parsed_response)

    def test_make_request_get_request(self):
        client = self._create_client()
        client.make_request('describeDirectors', 'get', '/get_request_path', {}, None)
        self.assertTrue(self.endpoint.make_request.called)
        params = self._get_request_params()
        self.assertEquals('get', params['method'])
        self.assertEquals('/get_request_path', params['url_path'])
        self.assertIsNone(params['body'])

    def test_make_request_absolute_url(self):
        client = self._create_client()
        client.make_request('describeDirectors', 'post', 'https://host/path', {}, None)
        self.assertTrue(self.endpoint.make_request.called)
        params = self._get_request_params()
        self.assertEquals('post', params['method'])
        self.assertEquals('https://host/path', params['url_path'])

    def test_make_request_no_redirects(self):
        client = self._create_client()
        client.make_request('describeDirectors', 'post', '/path', {}, None,
                            allow_redirects=False)
        self.assertTrue(self.endpoint.make_request.called)
        kwargs = self.endpoint.make_request.call_args[1]
        self.assertEquals({'allow_redirects': False}, kwargs)

    def test_make_request_error_with_no_redirects(self):
        client = self._create_client()
        error = {'error': {'code': 'test-code', 'message': 'test-message'}}
        mock_response = Mock(
            status_code=400,
            headers={'x-altus-request-id': '00000000-0000-0000-0000-000000000000'})
        self.endpoint.make_request.return_value = (mock_response, error)
        re = (
            'An error occurred: test-message '
            '\\(Status Code: 400; '
            'Error Code: test-code; '
            'Service: thunderhead; '
            'Operation: describeDirectors; '
            'Request ID: 00000000-0000-0000-0000-000000000000;\\)')
        with self.assertRaisesRegexp(ClientError, re) as e:
            client.make_request('describeDirectors',
                                'post',
                                '/directors/describeDirectors',
                                {},
                                None,
                                allow_redirects=False)
        self.assertEqual(e.exception.response['error']['code'], 'test-code')
        self.assertEqual(e.exception.response['error']['message'], 'test-message')

    def test_make_request_error(self):
        client = self._create_client()
        error = {'error': {'code': 'test-code', 'message': 'test-message'}}
        mock_response = Mock(
            status_code=400,
            headers={'x-altus-request-id': '00000000-0000-0000-0000-000000000000'})
        self.endpoint.make_request.return_value = (mock_response, error)
        re = (
            'An error occurred: test-message '
            '\\(Status Code: 400; '
            'Error Code: test-code; '
            'Service: thunderhead; '
            'Operation: describeDirectors; '
            'Request ID: 00000000-0000-0000-0000-000000000000;\\)')
        with self.assertRaisesRegexp(ClientError, re) as e:
            client.make_request('describeDirectors',
                                'post',
                                '/directors/describeDirectors',
                                {},
                                None)
        self.assertEqual(e.exception.response['error']['code'], 'test-code')
        self.assertEqual(e.exception.response['error']['message'], 'test-message')



# Copyright 2012-2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Modifications made by Cloudera are:
#     Copyright (c) 2016 Cloudera, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import logging
import re
import string

LOG = logging.getLogger('cdpcli.shorthand')
_EOF = object()


class _NamedRegex(object):
    def __init__(self, name, regex_str):
        self.name = name
        self.regex = re.compile(regex_str, re.UNICODE)

    def match(self, value):
        return self.regex.match(value)


class ShorthandParseError(Exception):
    def __init__(self, value, expected, actual, index):
        self.value = value
        self.expected = expected
        self.actual = actual
        self.index = index
        msg = self._construct_msg()
        super(ShorthandParseError, self).__init__(msg)

    def _construct_msg(self):
        consumed, remaining, num_spaces = self.value, '', self.index
        if '\n' in self.value[:self.index]:
            # If there's newlines in the consumed expression, we want
            # to make sure we're only counting the spaces
            # from the last newline:
            # foo=bar,\n
            # bar==baz
            #     ^
            last_newline = self.value[:self.index].rindex('\n')
            num_spaces = self.index - last_newline - 1
        if '\n' in self.value[self.index:]:
            # If there's newline in the remaining, divide value
            # into consumed and remainig
            # foo==bar,\n
            #     ^
            # bar=baz
            next_newline = self.index + self.value[self.index:].index('\n')
            consumed = self.value[:next_newline]
            remaining = self.value[next_newline:]
        msg = (
            "Expected: '%s', received: '%s' for input:\n"
            "%s\n"
            "%s"
            "%s"
        ) % (self.expected, self.actual, consumed,
             ' ' * num_spaces + '^', remaining)
        return msg


class ShorthandParser(object):
    """Parses shorthand syntax in the CLI.

    Note that this parser does not rely on any JSON models to control
    how to parse the shorthand syntax.

    """

    _SINGLE_QUOTED = _NamedRegex('singled quoted', r'\'(?:\\\\|\\\'|[^\'])*\'')
    _DOUBLE_QUOTED = _NamedRegex('double quoted', r'"(?:\\\\|\\"|[^"])*"')
    _START_WORD = u'\\!\\#-&\\(-\\+\\--\\<\\>-Z\\\\-z\u007c-\uffff'
    _FIRST_FOLLOW_CHARS = u'\\s\\!\\#-&\\(-\\+\\--\\\\\\^-\\|~-\uffff'
    _SECOND_FOLLOW_CHARS = u'\\s\\!\\#-&\\(-\\+\\--\\<\\>-\uffff'
    _ESCAPED_COMMA = '(\\\\,)'
    _FIRST_VALUE = _NamedRegex(
        'first',
        u'({escaped_comma}|[{start_word}])'
        u'({escaped_comma}|[{follow_chars}])*'.format(
            escaped_comma=_ESCAPED_COMMA,
            start_word=_START_WORD,
            follow_chars=_FIRST_FOLLOW_CHARS,
        ))
    _SECOND_VALUE = _NamedRegex(
        'second',
        u'({escaped_comma}|[{start_word}])'
        u'({escaped_comma}|[{follow_chars}])*'.format(
            escaped_comma=_ESCAPED_COMMA,
            start_word=_START_WORD,
            follow_chars=_SECOND_FOLLOW_CHARS,
        ))

    def __init__(self):
        self._tokens = []

    def parse(self, value):
        """Parse shorthand syntax.

        For example::

            parser = ShorthandParser()
            parser.parse('a=b')  # {'a': 'b'}
            parser.parse('a=b,c')  # {'a': ['b', 'c']}

        :tpye value: str
        :param value: Any value that needs to be parsed.

        :return: Parsed value, which will be a dictionary.
        """
        self._input_value = value
        self._index = 0
        return self._parameter()

    def _parameter(self):
        # parameter = keyval *("," keyval)
        params = {}
        params.update(self._keyval())
        while self._index < len(self._input_value):
            self._expect(',', consume_whitespace=True)
            keyval = self._keyval()
            for key in keyval.keys():
                if key in params:
                    LOG.warn('A value for the key "%s" was given more than '
                             'once. Only the last value listed will be used. '
                             'Separate multiple sets of key-value pairs with '
                             'spaces.', key)
            params.update(keyval)
        return params

    def _keyval(self):
        # keyval = key "=" [values]
        key = self._key()
        self._expect('=', consume_whitespace=True)
        values = self._values()
        return {key: values}

    def _key(self):
        # key = 1*(alpha / %x30-39 / %x5f / %x2e)  ; [a-zA-Z0-9\-_.]
        valid_chars = string.ascii_letters + string.digits + '-_.'
        start = self._index
        while not self._at_eof():
            if self._current() not in valid_chars:
                break
            self._index += 1
        return self._input_value[start:self._index]

    def _values(self):
        # values = csv-list / explicit-list / hash-literal
        if self._at_eof():
            return ''
        elif self._current() == '[':
            return self._explicit_list()
        elif self._current() == '{':
            return self._hash_literal()
        else:
            return self._csv_value()

    def _csv_value(self):
        # Supports either:
        # foo=bar     -> 'bar'
        #     ^
        # foo=bar,baz -> ['bar', 'baz']
        #     ^
        first_value = self._first_value()
        self._consume_whitespace()
        if self._at_eof() or self._input_value[self._index] != ',':
            return first_value
        self._expect(',', consume_whitespace=True)
        csv_list = [first_value]
        # Try to parse remaining list values.
        # It's possible we don't parse anything:
        # a=b,c=d
        #     ^-here
        # In the case above, we'll hit the ShorthandParser,
        # backtrack to the comma, and return a single scalar
        # value 'b'.
        while True:
            try:
                current = self._second_value()
                self._consume_whitespace()
                if self._at_eof():
                    csv_list.append(current)
                    break
                self._expect(',', consume_whitespace=True)
                csv_list.append(current)
            except ShorthandParseError:
                # Backtrack to the previous comma.
                # This can happen when we reach this case:
                # foo=a,b,c=d,e=f
                #     ^-start
                # foo=a,b,c=d,e=f
                #          ^-error, "expected ',' received '='
                # foo=a,b,c=d,e=f
                #        ^-backtrack to here.
                if self._at_eof():
                    raise
                self._backtrack_to(',')
                break
        if len(csv_list) == 1:
            # Then this was a foo=bar case, so we expect
            # this to parse to a scalar value 'bar', i.e
            # {"foo": "bar"} instead of {"bar": ["bar"]}
            return first_value
        return csv_list

    def _value(self):
        result = self._FIRST_VALUE.match(self._input_value[self._index:])
        if result is not None:
            consumed = self._consume_matched_regex(result)
            return consumed.replace('\\,', ',').rstrip()
        return ''

    def _explicit_list(self):
        # explicit-list = "[" [value *(",' value)] "]"
        self._expect('[', consume_whitespace=True)
        values = []
        while self._current() != ']':
            val = self._explicit_values()
            values.append(val)
            self._consume_whitespace()
            if self._current() != ']':
                self._expect(',')
                self._consume_whitespace()
        self._expect(']')
        return values

    def _explicit_values(self):
        # values = csv-list / explicit-list / hash-literal
        if self._current() == '[':
            return self._explicit_list()
        elif self._current() == '{':
            return self._hash_literal()
        else:
            return self._first_value()

    def _hash_literal(self):
        self._expect('{', consume_whitespace=True)
        keyvals = {}
        while self._current() != '}':
            key = self._key()
            self._expect('=', consume_whitespace=True)
            v = self._explicit_values()
            self._consume_whitespace()
            if self._current() != '}':
                self._expect(',')
                self._consume_whitespace()
            keyvals[key] = v
        self._expect('}')
        return keyvals

    def _first_value(self):
        # first-value = value / single-quoted-val / double-quoted-val
        if self._current() == "'":
            return self._single_quoted_value()
        elif self._current() == '"':
            return self._double_quoted_value()
        return self._value()

    def _single_quoted_value(self):
        # single-quoted-value = %x27 *(val-escaped-single) %x27
        # val-escaped-single  = %x20-26 / %x28-7F / escaped-escape /
        #                       (escape single-quote)
        return self._consume_quoted(self._SINGLE_QUOTED, escaped_char="'")

    def _consume_quoted(self, regex, escaped_char=None):
        value = self._must_consume_regex(regex)[1:-1]
        if escaped_char is not None:
            value = value.replace("\\%s" % escaped_char, escaped_char)
            value = value.replace("\\\\", "\\")
        return value

    def _double_quoted_value(self):
        return self._consume_quoted(self._DOUBLE_QUOTED, escaped_char='"')

    def _second_value(self):
        if self._current() == "'":
            return self._single_quoted_value()
        elif self._current() == '"':
            return self._double_quoted_value()
        else:
            consumed = self._must_consume_regex(self._SECOND_VALUE)
            return consumed.replace('\\,', ',').rstrip()

    def _expect(self, char, consume_whitespace=False):
        if consume_whitespace:
            self._consume_whitespace()
        if self._index >= len(self._input_value):
            raise ShorthandParseError(self._input_value, char,
                                      'EOF', self._index)
        actual = self._input_value[self._index]
        if actual != char:
            raise ShorthandParseError(self._input_value, char,
                                      actual, self._index)
        self._index += 1
        if consume_whitespace:
            self._consume_whitespace()

    def _must_consume_regex(self, regex):
        result = regex.match(self._input_value[self._index:])
        if result is not None:
            return self._consume_matched_regex(result)
        raise ShorthandParseError(self._input_value, '<%s>' % regex.name,
                                  '<none>', self._index)

    def _consume_matched_regex(self, result):
        start, end = result.span()
        v = self._input_value[self._index+start:self._index+end]
        self._index += (end - start)
        return v

    def _current(self):
        # If the index is at the end of the input value,
        # then _EOF will be returned.
        if self._index < len(self._input_value):
            return self._input_value[self._index]
        return _EOF

    def _at_eof(self):
        return self._index >= len(self._input_value)

    def _backtrack_to(self, char):
        while self._index >= 0 and self._input_value[self._index] != char:
            self._index -= 1

    def _consume_whitespace(self):
        while self._current() != _EOF and self._current() in string.whitespace:
            self._index += 1


class ModelVisitor(object):
    def visit(self, params, model):
        self._visit({}, model, '', params)

    def _visit(self, parent, shape, name, value):
        method = getattr(self, '_visit_%s' % shape.type_name,
                         self._visit_scalar)
        method(parent, shape, name, value)

    def _visit_object(self, parent, shape, name, value):
        if not isinstance(value, dict):
            return
        for member_name, member_shape in shape.members.items():
            self._visit(value, member_shape, member_name,
                        value.get(member_name))

    def _visit_map(self, parent, shape, name, value):
        if not isinstance(value, dict):
            return
        for k, v in value.items():
            self._visit(value, shape.value, k, v)

    def _visit_array(self, parent, shape, name, value):
        if not isinstance(value, list):
            return
        for i, element in enumerate(value):
            self._visit(value, shape.member, i, element)

    def _visit_scalar(self, parent, shape, name, value):
        pass


class BackCompatVisitor(ModelVisitor):
    def _visit_array(self, parent, shape, name, value):
        if not isinstance(value, list):
            # Convert a -> [a] because they specified
            # "foo=bar", but "bar" should really be ["bar"].
            if value is not None:
                parent[name] = [value]
        else:
            return super(BackCompatVisitor, self)._visit_array(
                parent, shape, name, value)

    def _visit_scalar(self, parent, shape, name, value):
        if value is None:
            return
        type_name = shape.type_name
        if type_name in ['integer', 'long']:
            parent[name] = int(value)
        elif type_name in ['double', 'float']:
            parent[name] = float(value)
        elif type_name == 'boolean':
            # We want to make sure we only set a value
            # only if "true"/"false" is specified.
            if value.lower() == 'true':
                parent[name] = True
            elif value.lower() == 'false':
                parent[name] = False



# Copyright 2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Modifications made by Cloudera are:
#     Copyright (c) 2016 Cloudera, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from cdpcli.exceptions import PaginationError
from cdpcli.extensions import paginate
from cdpcli.help import OperationHelpCommand
from cdpcli.model import OperationModel
import mock
from mock import Mock
from tests import unittest


class TestPaginateBase(unittest.TestCase):

    def setUp(self):
        self.operation_model = mock.Mock()
        self.foo_param = mock.Mock()
        self.foo_param.name = 'Foo'
        self.foo_param.type_name = 'string'
        self.bar_param = mock.Mock()
        self.bar_param.type_name = 'string'
        self.bar_param.name = 'Bar'
        self.bartoo_param = mock.Mock()
        self.bartoo_param.type_name = 'string'
        self.bartoo_param.name = 'BarToo'
        self.params = [self.foo_param, self.bar_param, self.bartoo_param]
        self.operation_model.input_shape.members = {"Foo": self.foo_param,
                                                    "Bar": self.bar_param,
                                                    "BarToo": self.bartoo_param}
        self.operation_model.can_paginate = True
        self.operation_model.paging_input_token = "Foo"
        self.operation_model.paging_page_size = "Bar"
        self.operation_model.paging_default_max_items = 100

    def validate_page_param(self, param, is_shadowing=False):
        self.assertIsInstance(param, paginate.PageArgument)
        if is_shadowing:
            self.assertIsNotNone(param.shadowed_argument)
        else:
            self.assertIsNone(param.shadowed_argument)


class TestArgumentTableModifications(TestPaginateBase):

    def test_customize_arg_table(self):
        argument_table = {
            'foo': mock.Mock(),
            'bar': mock.Mock(),
            'bar-too': mock.Mock()
        }
        paginate.unify_paging_params(self.operation_model, argument_table)
        # We should mark the built in input_token as 'hidden'.
        self.assertTrue(argument_table['foo']._UNDOCUMENTED)
        # Also need to hide the page size.
        self.assertTrue(argument_table['bar']._UNDOCUMENTED)
        # Also need to hide the max size.
        self.assertTrue(argument_table['bar-too']._UNDOCUMENTED)
        # We also need to inject startin-token and max-items.
        self.assertIn('starting-token', argument_table)
        self.assertIn('max-items', argument_table)
        self.assertIn('page-size', argument_table)

        self.validate_page_param(argument_table['starting-token'])
        self.validate_page_param(argument_table['page-size'])
        self.validate_page_param(argument_table['max-items'])

    def test_operation_with_no_paginate(self):
        self.operation_model.can_paginate = False
        argument_table = {
            'foo': 'FakeArgObject',
            'bar': 'FakeArgObject',
        }
        starting_table = argument_table.copy()
        paginate.unify_paging_params(self.operation_model, argument_table)
        self.assertEqual(starting_table, argument_table)


class TestHelpDocumentationModifications(TestPaginateBase):
    def test_injects_pagination_help_text(self):
        help_command = OperationHelpCommand(
            Mock(), Mock(), 'foo', Mock())
        help_command.obj = Mock(OperationModel)
        help_command.obj.can_paginate = True
        help_command.obj.name = 'foo'
        paginate.add_paging_description(help_command)
        self.assertIn('``foo`` is a paginated operation. Multiple API',
                      help_command.doc.getvalue().decode())

    def test_does_not_inject_when_no_pagination(self):
        help_command = OperationHelpCommand(
            Mock(), Mock(), 'foo', Mock())
        help_command.obj = Mock(OperationModel)
        help_command.obj.can_paginate = False
        help_command.obj.name = 'foo'
        paginate.add_paging_description(help_command)
        self.assertNotIn('``foo`` is a paginated operation',
                         help_command.doc.getvalue().decode())


class TestShouldEnablePagination(TestPaginateBase):
    def setUp(self):
        super(TestShouldEnablePagination, self).setUp()
        self.parsed_globals = mock.Mock()
        self.parsed_args = mock.Mock()
        self.parsed_args.starting_token = None
        self.parsed_args.page_size = None
        self.parsed_args.max_items = None
        self.argument_table = {
            'foo': mock.Mock(),
            'bar': mock.Mock(),
            'bar-too': mock.Mock()
        }

    def test_should_not_enable_pagination(self):
        # Here the user has specified a manual pagination argument,
        # so we should turn pagination off.
        self.parsed_globals.paginate = True
        # Corresponds to --bar 10
        self.parsed_args.foo = None
        self.parsed_args.bar = 10
        self.parsed_args.bar_too = None
        paginate.check_should_enable_pagination(self.argument_table,
                                                self.operation_model,
                                                self.parsed_args,
                                                self.parsed_globals)
        # We should have turned paginate off because the
        # user specified --bar 10
        self.assertFalse(self.parsed_globals.paginate)

    def test_should_enable_pagination_with_no_args(self):
        self.parsed_globals.paginate = True
        # Corresponds to not specifying --foo nor --bar
        self.parsed_args.foo = None
        self.parsed_args.bar = None
        self.parsed_args.bar_too = None
        paginate.check_should_enable_pagination(self.argument_table,
                                                self.operation_model,
                                                self.parsed_args,
                                                self.parsed_globals)
        self.assertTrue(self.parsed_globals.paginate)

    def test_default_to_pagination_on_when_ambiguous(self):
        argument_table = {
            'foo': mock.Mock(),
            'max-times': mock.Mock()
        }
        self.parsed_globals.paginate = True
        # Here the user specifies --max-items 10 This is ambiguous because the
        # input_token also contains 'max-items'.  Should we assume they want
        # pagination turned off or should we assume that this is the normalized
        # --max-items?
        # Will we default to assuming they meant the normalized
        # --max-items.
        self.parsed_args.foo = None
        self.parsed_args.bar = None
        self.parsed_args.bar_too = None
        self.parsed_args.max_items = 10
        paginate.check_should_enable_pagination(argument_table,
                                                self.operation_model,
                                                self.parsed_args,
                                                self.parsed_globals)
        self.assertTrue(self.parsed_globals.paginate,
                        "Pagination was not enabled.")

    def test_shadowed_args_are_replaced_when_pagination_turned_off(self):
        operation_model = Mock(OperationModel)
        operation_model.can_paginate = True
        operation_model.paging_input_token = "StartingToken"
        operation_model.paging_page_size = "Bar"
        operation_model.paging_default_max_items = 100
        self.parsed_globals.paginate = True
        # Corresponds to --bar 10
        self.parsed_args.foo = None
        self.parsed_args.bar = 10
        self.parsed_args.bar_too = None

        argument_table = {
            'starting-token': mock.sentinel.FOO_ARG,
            'bar': mock.sentinel.BAR_ARG,
            'bar-too': mock.sentinel.BAR_TOO_ARG
        }
        paginate.unify_paging_params(operation_model, argument_table)
        self.validate_page_param(argument_table['starting-token'], True)
        paginate.check_should_enable_pagination(argument_table,
                                                operation_model,
                                                self.parsed_args,
                                                self.parsed_globals)

        # We should have turned paginate off because the
        # user specified --bar 10
        self.assertFalse(self.parsed_globals.paginate)
        self.assertEqual(argument_table['starting-token'], mock.sentinel.FOO_ARG)

    def test_shadowed_args_are_replaced_when_pagination_set_off(self):
        operation_model = Mock(OperationModel)
        operation_model.can_paginate = True
        operation_model.paging_input_token = "Foo"
        operation_model.paging_page_size = "page-size"
        operation_model.paging_default_max_items = 100
        self.parsed_globals.paginate = False
        # Corresponds to --foo 10
        self.parsed_args.foo = 10
        self.parsed_args.bar = None
        argument_table = {
            'foo': mock.sentinel.FOO_ARG,
            'page-size': mock.sentinel.BAR_ARG,
            'bar-too': mock.sentinel.BAR_TOO_ARG
        }
        paginate.unify_paging_params(operation_model, argument_table)
        self.validate_page_param(argument_table['page-size'], True)
        paginate.check_should_enable_pagination(argument_table,
                                                operation_model,
                                                self.parsed_args,
                                                self.parsed_globals)
        # We should have turned paginate off because the
        # user specified --bar 10
        self.assertFalse(self.parsed_globals.paginate)
        self.assertEqual(argument_table['page-size'], mock.sentinel.BAR_ARG)


class TestEnsurePagingParamsNotSet(TestPaginateBase):
    def setUp(self):
        super(TestEnsurePagingParamsNotSet, self).setUp()
        self.parsed_args = mock.Mock()

        self.parsed_args.starting_token = None
        self.parsed_args.page_size = None
        self.parsed_args.max_items = None

    def test_pagination_params_raise_error_with_no_paginate(self):
        self.parsed_args.max_items = 100

        with self.assertRaises(PaginationError):
            paginate.ensure_paging_params_not_set({}, self.parsed_args)

    def test_can_handle_missing_page_size(self):
        # Not all pagination operations have a page_size.
        del self.parsed_args.page_size
        self.assertIsNone(paginate.ensure_paging_params_not_set(
            {}, self.parsed_args))



# Copyright 2016 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Modifications made by Cloudera are:
#     Copyright (c) 2016 Cloudera, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from cdpcli.compat import six
from cdpcli.extensions.configure.get import ConfigureGetCommand
from tests import unittest
from tests.unit import FakeContext


class TestConfigureGetCommand(unittest.TestCase):

    def test_configure_get_command(self):
        context = FakeContext({})
        context.config['region'] = 'us-west-2'
        stream = six.StringIO()
        config_get = ConfigureGetCommand(stream)
        config_get(context, args=['region'], parsed_globals=None)
        rendered = stream.getvalue()
        self.assertEqual(rendered.strip(), 'us-west-2')

    def test_configure_get_command_no_exist(self):
        no_vars_defined = {}
        context = FakeContext(no_vars_defined)
        stream = six.StringIO()
        config_get = ConfigureGetCommand(stream)
        rc = config_get(context, args=['region'], parsed_globals=None)
        rendered = stream.getvalue()
        # If a config value does not exist, we don't print any output.
        self.assertEqual(rendered, '')
        # And we exit with an rc of 1.
        self.assertEqual(rc, 1)

    def test_dotted_get(self):
        context = FakeContext({})
        context.full_config = {'preview': {'foo': 'true'}}
        stream = six.StringIO()
        config_get = ConfigureGetCommand(stream)
        config_get(context, args=['preview.foo'], parsed_globals=None)
        rendered = stream.getvalue()
        self.assertEqual(rendered.strip(), 'true')

    def test_predefined_section_with_profile(self):
        # Test that we retrieve the predefined section config var even if it's
        # under a profile.
        context = FakeContext({})
        context.full_config = {'profiles': {'thu-dev': {
            'thu': {'instance_profile': 'my_ip'}}, 'preview': {'foo': 'true'}}}
        stream = six.StringIO()
        config_get = ConfigureGetCommand(stream)
        config_get(context, args=['preview.foo'], parsed_globals=None)
        rendered = stream.getvalue()
        self.assertEqual(rendered.strip(), 'true')

    def test_dotted_get_with_profile(self):
        context = FakeContext({})
        context.full_config = {'profiles': {'thu-dev': {
            'thu': {'instance_profile': 'my_ip'}}}}
        context.config = {'thu': {'instance_profile': 'my_ip'}}
        stream = six.StringIO()
        config_get = ConfigureGetCommand(stream)
        config_get(context,
                   args=['thu-dev.thu.instance_profile'],
                   parsed_globals=None)
        rendered = stream.getvalue()
        self.assertEqual(rendered.strip(), 'my_ip')

    def test_get_from_profile(self):
        context = FakeContext({})
        context.full_config = \
            {'profiles': {'testing': {'cdp_access_key_id': 'access_key'}}}
        stream = six.StringIO()
        config_get = ConfigureGetCommand(stream)
        config_get(context,
                   args=['profile.testing.cdp_access_key_id'],
                   parsed_globals=None)
        rendered = stream.getvalue()
        self.assertEqual(rendered.strip(), 'access_key')

    def test_get_nested_attribute(self):
        context = FakeContext({})
        context.full_config = {
            'profiles': {'testing': {'s3': {'signature_version': 's3v4'}}}}
        stream = six.StringIO()
        config_get = ConfigureGetCommand(stream)
        config_get(context,
                   args=['profile.testing.s3.signature_version'],
                   parsed_globals=None)
        rendered = stream.getvalue()
        self.assertEqual(rendered.strip(), 's3v4')

    def test_get_nested_attribute_from_default(self):
        context = FakeContext({})
        context.full_config = {
            'profiles': {'default': {'s3': {'signature_version': 's3v4'}}}}
        stream = six.StringIO()
        config_get = ConfigureGetCommand(stream)
        config_get(context,
                   args=['default.s3.signature_version'],
                   parsed_globals=None)
        rendered = stream.getvalue()
        self.assertEqual(rendered.strip(), 's3v4')

    def test_get_nested_attribute_from_default_does_not_exist(self):
        context = FakeContext({})
        context.full_config = {'profiles': {}}
        stream = six.StringIO()
        config_get = ConfigureGetCommand(stream)
        config_get(context,
                   args=['default.s3.signature_version'],
                   parsed_globals=None)
        rendered = stream.getvalue()
        self.assertEqual(rendered.strip(), '')

    def test_dotted_not_in_full_config_get(self):
        context = FakeContext({})
        context.full_config = {
            'profiles': {'dev': {'someconf': {'foobar': 'true'}}}}
        stream = six.StringIO()
        context.variables['profile'] = 'dev'
        config_get = ConfigureGetCommand(stream)
        config_get(context,
                   args=['someconf.foobar'],
                   parsed_globals=None)
        rendered = stream.getvalue()
        self.assertEqual(rendered.strip(), 'true')



# Copyright (c) 2021 Cloudera, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from cdpcli.exceptions import WorkloadServiceDiscoveryError
from cdpcli.extensions.workload import register, WorkloadServiceDiscovery
from mock import Mock
from tests import unittest


class TestWorkloadExtension(unittest.TestCase):

    def test_workload_extension_register(self):
        operation_callers = [Mock()]
        register(operation_callers, None)
        self.assertEqual(2, len(operation_callers))
        self.assertIsInstance(operation_callers[0], WorkloadServiceDiscovery)

    def test_service_discovery_df(self):
        response = {'endpointUrl': 'http://workload-host/path',
                    'token': 'workload-access-token'}
        generate_workload_auth_token = Mock(return_value=response)
        iam_client = Mock()
        iam_client.generate_workload_auth_token = generate_workload_auth_token
        client_creator = Mock(return_value=iam_client)
        operation_model = Mock(name='mockOp')
        operation_model.service_model = Mock(service_name='dfworkload')
        parameters = {'environmentCrn': 'mockCrn'}
        parsed_args = Mock()
        parsed_globals = Mock(endpoint_url=None, access_token=None)

        workload_service_discovery = WorkloadServiceDiscovery()
        ret = workload_service_discovery.invoke(client_creator,
                                                operation_model,
                                                parameters,
                                                parsed_args,
                                                parsed_globals)
        self.assertTrue(ret)

        self.assertTrue(client_creator.called)
        args, kargs = client_creator.call_args
        service_name, = args
        self.assertEqual('iam', service_name)

        self.assertTrue(generate_workload_auth_token.called)
        args, kargs = generate_workload_auth_token.call_args
        self.assertEqual({'workloadName': 'DF', 'environmentCrn': 'mockCrn'}, kargs)

        self.assertEqual('http://workload-host/', parsed_globals.endpoint_url)
        self.assertEqual('Bearer workload-access-token', parsed_globals.access_token)

    def test_service_discovery_no_response(self):
        iam_client = Mock()
        iam_client.generate_workload_auth_token = Mock(return_value={})
        client_creator = Mock(return_value=iam_client)
        operation_model = Mock(name='mockOp')
        operation_model.service_model = Mock(service_name='dfworkload')
        parameters = {'environmentCrn': 'mockCrn'}
        parsed_args = Mock()
        parsed_globals = Mock(endpoint_url=None, access_token=None)

        workload_service_discovery = WorkloadServiceDiscovery()
        ret = workload_service_discovery.invoke(client_creator,
                                                operation_model,
                                                parameters,
                                                parsed_args,
                                                parsed_globals)
        self.assertTrue(ret)
        self.assertTrue(iam_client.generate_workload_auth_token.called)
        self.assertIsNone(parsed_globals.endpoint_url)
        self.assertIsNone(parsed_globals.access_token)

    def test_service_discovery_skip_by_access_token(self):
        client_creator = Mock()
        operation_model = Mock(name='mockOp')
        operation_model.service_model = Mock(service_name='dfworkload')
        parsed_globals = Mock()
        parsed_globals.endpoint_url = None
        parsed_globals.access_token = 'Bearer ABC'
        workload_service_discovery = WorkloadServiceDiscovery()
        workload_service_discovery.invoke(client_creator,
                                          operation_model,
                                          {'environmentCrn': 'mockCrn'},
                                          Mock(),
                                          parsed_globals)
        self.assertFalse(client_creator.called)
        self.assertIsNone(parsed_globals.endpoint_url)

    def test_service_discovery_no_service_name(self):
        client_creator = Mock()
        operation_model = Mock(name='mockOp')
        operation_model.service_model = Mock(service_name=None)
        parsed_globals = Mock(endpoint_url=None, access_token=None)
        workload_service_discovery = WorkloadServiceDiscovery()

        with self.assertRaises(WorkloadServiceDiscoveryError) as context:
            workload_service_discovery.invoke(client_creator,
                                              operation_model,
                                              {'environmentCrn': 'mockCrn'},
                                              Mock(),
                                              parsed_globals)
        self.assertFalse(client_creator.called)
        self.assertIn('Workload service-discovery error: Missing service name.',
                      str(context.exception))

        operation_model = Mock(name='mockOp')
        operation_model.service_model = Mock(service_name='')
        with self.assertRaises(WorkloadServiceDiscoveryError) as context:
            workload_service_discovery.invoke(client_creator,
                                              operation_model,
                                              {'environmentCrn': 'mockCrn'},
                                              Mock(),
                                              parsed_globals)
        self.assertFalse(client_creator.called)
        self.assertIn('Workload service-discovery error: Missing service name.',
                      str(context.exception))

    def test_service_discovery_unknown_service_name(self):
        client_creator = Mock()
        operation_model = Mock(name='mockOp')
        operation_model.service_model = Mock(service_name='invalid')
        parsed_globals = Mock(endpoint_url=None, access_token=None)
        workload_service_discovery = WorkloadServiceDiscovery()

        with self.assertRaises(WorkloadServiceDiscoveryError) as context:
            workload_service_discovery.invoke(client_creator,
                                              operation_model,
                                              {'environmentCrn': 'mockCrn'},
                                              Mock(),
                                              parsed_globals)
        self.assertFalse(client_creator.called)
        self.assertIn('Workload service-discovery error: '
                      'Unknown service name \'invalid\'.',
                      str(context.exception))

    def test_service_discovery_no_environment_crn(self):
        client_creator = Mock()
        operation_model = Mock(name='mockOp')
        operation_model.service_model = Mock(service_name='dfworkload')
        parsed_globals = Mock(endpoint_url=None, access_token=None)
        workload_service_discovery = WorkloadServiceDiscovery()

        with self.assertRaises(WorkloadServiceDiscoveryError) as context:
            workload_service_discovery.invoke(client_creator,
                                              operation_model,
                                              {},
                                              Mock(),
                                              parsed_globals)
        self.assertFalse(client_creator.called)
        self.assertIn('Workload service-discovery error: Missing environment CRN.',
                      str(context.exception))

        with self.assertRaises(WorkloadServiceDiscoveryError) as context:
            workload_service_discovery.invoke(client_creator,
                                              operation_model,
                                              {'environmentCrn': ''},
                                              Mock(),
                                              parsed_globals)
        self.assertFalse(client_creator.called)
        self.assertIn('Workload service-discovery error: Missing environment CRN.',
                      str(context.exception))

        with self.assertRaises(WorkloadServiceDiscoveryError) as context:
            workload_service_discovery.invoke(client_creator,
                                              operation_model,
                                              {'environmentCrn': None},
                                              Mock(),
                                              parsed_globals)
        self.assertFalse(client_creator.called)
        self.assertIn('Workload service-discovery error: Missing environment CRN.',
                      str(context.exception))



# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Modifications made by Cloudera are:
#     Copyright (c) 2019 Cloudera, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from cdpcli.exceptions import ProfileNotFound


RSA_KEY = \
    """-----BEGIN PRIVATE KEY-----
MIIEvQIBADANBgkqhkiG9w0BAQEFAASCBKcwggSjAgEAAoIBAQCsdjjYZKA3pqzj
XXZq2NidIg9UE3oGajYXftpiTVS3vJtfrK38202r3YbrMdAtzSm5BzVHU74C2XVX
RWrCZ9Js9Dh3RYta7uqh9b1PNdJIP+/fUBJZs5eJ+oMkTbmtgFznprUi7RcrU9Pb
k9ByIqnaJv6Dw/1SvsbwAIjbAyqNXP0WJvFc0hznHu5Ok3US8uz+fdIbalPSEKgN
6fmYmhabaGUbXmMrUevtIL9BRUY7/3vCR+vm+6UKfQqS4knMYLZN7NjIl4t/WvTX
RfuhObpgaD1c3S0LmFwFeTAmguG2tbSNDzkzG9tN3zuVxtL0+Sy/I3f+kHEVkDQW
CnnGRhBRAgMBAAECggEAbgILSJ0HRfhfl7hqbMVdhv3O4UZ7M9RUJLCaBNJnE7yP
L3wqj3wkE1j/Us83h7+yuX/LkG/uaErl+oEhFFi9dRpjWlFWDu8PY7goxXoDZGrE
S6H70pQoOa8+L84UfoO+v1UrfdfWS6DxJsMm12cdCTaOauZ9lGZ052qv4WQnpHt7
K9asyrsGSE4W1aKHIPUGbreOfwELDH/PkLLHsbVnC5PG6jpQaJHUEoebkQkVW7Ru
mG4AwH26HTNoVF5YPGSy0BRHab+0mrh4X2HHBTyzKEWQUYlWQlfW9iekmkdbrELr
adhqanguSwVUm6jrn4CRn6M2Wn55/Jv8LytlGUpBcQKBgQDAcPg74uwcz4Yoc6OM
uKVCxqwL5fOvxpVSc4+u07KCbX6tPsEXvCiPs30/a71xwcP42QBt9aLqRd8C+z99
ri36LNNLRc9/SH3TmezzXhuBK3nkX3pT6zJuiX+T5WrUueOyANUm0d9cvQsrH6uF
LtK9rXG4WKPH223QqBEPhv4yqwKBgQDla/mGm620PATZ6yUIfmJLLZF3fWiqekfk
2R6kyUhUm672mzqi1xTOByfobMjsel5UXW6gzwm9qkBG47OrvjINyq+ja/UOO4kq
ZtqlgNkL4nN5y2GTXbJrG7eabgsB3nq1+IvDC+dFTPqSuJoYfoZZtT58eMAHGTmh
Ka8ehzPo8wKBgExyFAomIMlpHtAe789M4klehqXLWTxwVI0GXwOCER2CxZmonigB
lNNQ5+YztHPmFyVZfrQvqeIKk4aprBUPBjClceIq/zx+3Y0bTmd28NIlJSy1SPDh
M415jXaA4ilTFsJ1Vjcvk91RM4iT8hzb9tdmeRBUFeuknUEQIobah0w1AoGBAKpW
f97sqYz/Xw65ozZqN+rfe3j/eP3SapzEhBcPh4+iQ8a/vEp5bO4HrB7K3meN94mm
EWR+NBpJVQ4NNDKYtas9ySiKGFmn5JDB6yckwoIrcVeFpP34fGdAHhMgDzYlDHEd
iA+aP+1ZWVYkj+0NzAzBIBLkyJa8qOg6/dWpxuX3AoGAYW0qU+s8bumhvxPEMbfQ
zU0IHNaprGxO+yZuMvdyfxcJVuFQf22mujPI8GluXyfA5i2IZjScNf5Awccu8HS5
mzJp12+5oAZ+tJE1scpVoaVdZloY8XA1u0aCt2BhZavjIizTyzY4L8clo2iGIO1x
yj5v7z/1wWTorT4w3/IHcwM=
-----END PRIVATE KEY-----"""

ED25519_KEY = 'VHswlNpwJnMpuXhIq7LYDfzs+R9pHvVLqgBGbhcnbSE='

ECDSA_KEY = \
    """-----BEGIN PRIVATE KEY-----
MIH3AgEAMBAGByqGSM49AgEGBSuBBAAjBIHfMIHcAgEBBEIB/VhgNb6ccDq9hV/s
TD8f1hSZo2Y7A7zZbCK0418vU0HYOi4Ic8TKjKIYw8cMQN7+2fGLQTkGezgu9NBw
gzR/LGKgBwYFK4EEACOhgYkDgYYABAGxMOPpk3m7wgXPRHBCSpW9as4tuCv38UJQ
37xSJqmim6LlF6s/eFRpRktNO98fLuAvAFt+97iYwOfhix4h/uVCSgFR0xloJtMe
MEQft1ColfYUlGSiNuMsGyugK/ilkmulX52Y7z3RY6hxlFAegHI8d9uzeVqUL+KO
1xtgNBDXVMnQCA==
-----END PRIVATE KEY-----"""


class FakeContext(object):

    def __init__(self,
                 all_variables,
                 profile_does_not_exist=False,
                 config_file_vars={},
                 environment_vars={},
                 credentials=None):
        self.variables = all_variables
        self.profile_does_not_exist = profile_does_not_exist
        self.config = {}
        self.config_file_vars = config_file_vars
        self.environment_vars = environment_vars
        self._credentials = credentials
        self.profile = None
        self.effective_profile = None
        # This lets us use the FakeContext as both context and "client_creator"
        self.context = self

    def get_credentials(self, parsed_globals=None):
        return self._credentials

    def get_scoped_config(self):
        if self.profile_does_not_exist:
            raise ProfileNotFound(profile='foo')
        return self.config

    def get_config_variable(self, name, methods=None):
        if name == 'credentials_file':
            # The credentials_file var doesn't require a
            # profile to exist.
            return '~/fake_credentials_filename'
        if self.profile_does_not_exist and not name == 'config_file':
            raise ProfileNotFound(profile='foo')
        if methods is not None:
            if 'env' in methods:
                return self.environment_vars.get(name)
            elif 'config' in methods:
                return self.config_file_vars.get(name)
        else:
            return self.variables.get(name)

    def _build_profile_map(self):
        if self.full_config is None:
            return None
        return self.full_config['profiles']



# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Modifications made by Cloudera are:
#     Copyright (c) 2016 Cloudera, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from cdpcli.arguments import CustomArgument


class OverrideRequiredArgsArgument(CustomArgument):
    """
    An argument that if specified makes all other arguments not required
    By not required, it refers to not having an error thrown when the
    parser does not find an argument that is required on the command line.
    To obtain this argument's property of ignoring required arguments,
    subclass from this class and fill out the ``ARG_DATA`` parameter as
    described below. Note this class is really only useful for subclassing.
    """

    # ``ARG_DATA`` follows the same format as a member of ``ARG_TABLE`` in
    # ``BasicCommand`` class as specified in
    # ``cdpcli/extensions/commands.py``.
    #
    # For example, an ``ARG_DATA`` variable would be filled out as:
    #
    # ARG_DATA =
    # {'name': 'my-argument',
    #  'help_text': 'This is argument ensures the argument is specified'
    #               'no other arguments are required'}
    ARG_DATA = {'name': 'no-required-args'}

    def __init__(self):
        super(OverrideRequiredArgsArgument, self).__init__(**self.ARG_DATA)

    def override_required_args(self, argument_table, args, **kwargs):
        name_in_cmdline = '--' + self.name
        # Set all ``Argument`` objects in ``argument_table`` to not required
        # if this argument's name is present in the command line.
        if name_in_cmdline in args:
            for arg_name in argument_table.keys():
                argument_table[arg_name].required = False

    def invoke(self,
               client_creator,
               operation_model,
               parameters,
               parsed_args,
               parsed_globals):
        """
        Invokes the argument handler. Returns 'True' if other operation call
        handlers should be invoked after it, 'False' to indicate that no other
        invocations should be made.
        """
        raise NotImplementedError("Derived classes must implement this method")



# Copyright 2016 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Modifications made by Cloudera are:
#     Copyright (c) 2016 Cloudera, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import os

from cdpcli import CDP_ACCESS_KEY_ID_KEY_NAME, \
                   CDP_ACCESS_TOKEN_KEY_NAME, \
                   CDP_PRIVATE_KEY_KEY_NAME
from cdpcli.extensions.configure.set import ConfigureSetCommand
import mock
from tests import unittest
from tests.unit import FakeContext


class TestConfigureSetCommand(unittest.TestCase):

    def setUp(self):
        self.context = FakeContext({'config_file': 'myconfigfile'})
        self.fake_credentials_filename = os.path.expanduser(
            '~/fake_credentials_filename')
        self.context.profile = None
        self.config_writer = mock.Mock()

    def test_configure_set_command(self):
        set_command = ConfigureSetCommand(self.config_writer)
        set_command(self.context,
                    args=['region', 'us-west-2'],
                    parsed_globals=None)
        self.config_writer.update_config.assert_called_with(
            {'__section__': 'default', 'region': 'us-west-2'}, 'myconfigfile')

    def test_configure_set_command_dotted(self):
        set_command = ConfigureSetCommand(self.config_writer)
        set_command(self.context,
                    args=['preview.foo', 'true'],
                    parsed_globals=None)
        self.config_writer.update_config.assert_called_with(
            {'__section__': 'preview', 'foo': 'true'}, 'myconfigfile')

    def test_configure_set_command_dotted_with_default_profile(self):
        self.context.variables['profile'] = 'default'
        set_command = ConfigureSetCommand(self.config_writer)
        set_command(self.context,
                    args=['foo.instance_profile', 'my_ip_foo'],
                    parsed_globals=None)
        self.config_writer.update_config.assert_called_with(
            {'__section__': 'default',
             'foo': {'instance_profile': 'my_ip_foo'}}, 'myconfigfile')

    def test_configure_set_command_dotted_with_profile(self):
        self.context.profile = 'thu-dev'
        set_command = ConfigureSetCommand(self.config_writer)
        set_command(self.context,
                    args=['thu.instance_profile', 'my_ip_thu'],
                    parsed_globals=None)
        self.config_writer.update_config.assert_called_with(
            {'__section__': 'profile thu-dev', 'thu':
                {'instance_profile': 'my_ip_thu'}}, 'myconfigfile')

    def test_configure_set_with_profile(self):
        self.context.profile = 'testing'
        set_command = ConfigureSetCommand(self.config_writer)
        set_command(self.context,
                    args=['region', 'us-west-2'],
                    parsed_globals=None)
        self.config_writer.update_config.assert_called_with(
            {'__section__': 'profile testing', 'region': 'us-west-2'},
            'myconfigfile')

    def test_configure_set_triple_dotted(self):
        # cdp configure set default.s3.signature_version s3v4
        set_command = ConfigureSetCommand(self.config_writer)
        set_command(self.context,
                    args=['default.s3.signature_version', 's3v4'],
                    parsed_globals=None)
        self.config_writer.update_config.assert_called_with(
            {'__section__': 'default', 's3': {'signature_version': 's3v4'}},
            'myconfigfile')

    def test_configure_set_with_profile_nested(self):
        # cdp configure set default.s3.signature_version s3v4
        set_command = ConfigureSetCommand(self.config_writer)
        set_command(self.context,
                    args=['profile.foo.s3.signature_version', 's3v4'],
                    parsed_globals=None)
        self.config_writer.update_config.assert_called_with(
            {'__section__': 'profile foo',
             's3': {'signature_version': 's3v4'}}, 'myconfigfile')

    def test_access_key_written_to_shared_credentials_file(self):
        set_command = ConfigureSetCommand(self.config_writer)
        set_command(self.context,
                    args=[CDP_ACCESS_KEY_ID_KEY_NAME, 'foo'],
                    parsed_globals=None)
        self.config_writer.update_config.assert_called_with(
            {'__section__': 'default',
             CDP_ACCESS_KEY_ID_KEY_NAME: 'foo'}, self.fake_credentials_filename)

    def test_private_key_written_to_shared_credentials_file(self):
        set_command = ConfigureSetCommand(self.config_writer)
        set_command(self.context,
                    args=[CDP_PRIVATE_KEY_KEY_NAME, 'foo'],
                    parsed_globals=None)
        self.config_writer.update_config.assert_called_with(
            {'__section__': 'default',
             CDP_PRIVATE_KEY_KEY_NAME: 'foo'}, self.fake_credentials_filename)

    def test_access_token_written_to_shared_credentials_file(self):
        set_command = ConfigureSetCommand(self.config_writer)
        set_command(self.context,
                    args=[CDP_ACCESS_TOKEN_KEY_NAME, 'foo'],
                    parsed_globals=None)
        self.config_writer.update_config.assert_called_with(
            {'__section__': 'default',
             CDP_ACCESS_TOKEN_KEY_NAME: 'foo'}, self.fake_credentials_filename)

    def test_access_key_written_to_shared_credentials_file_profile(self):
        set_command = ConfigureSetCommand(self.config_writer)
        set_command(self.context,
                    args=['profile.foo.cdp_access_key_id', 'bar'],
                    parsed_globals=None)
        self.config_writer.update_config.assert_called_with(
            {'__section__': 'foo',
             CDP_ACCESS_KEY_ID_KEY_NAME: 'bar'}, self.fake_credentials_filename)



# Licensed to Cloudera, Inc. under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  Cloudera, Inc. licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Extensions

A central place for extensions for our cdpcli. Extensions are loaded after
the known service models are loaded and can extend or modified one of these
services or extend the cdpcli.
"""



# Copyright 2021 Cloudera, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.


import logging

from cdpcli.exceptions import RedirectExtensionError
from cdpcli.utils import is_absolute_url


LOG = logging.getLogger('cdpcli.extensions.redirect')


def register(operation_callers, operation_model):
    """
    Register an extension to run before or after the CLI command.
    To replace the original CLI caller:
    * operation_callers.insert(0, ReplacementCaller())
    * return False by the ReplacementCaller.invoke(...)
    """
    operation_callers.insert(0, Redirect())


class Redirect(object):
    def invoke(self,
               client_creator,
               operation_model,
               parameters,
               parsed_args,
               parsed_globals):
        service_name = operation_model.service_model.service_name
        operation_name = operation_model.name
        client = client_creator(service_name)
        redirect_url = self._find_redirect_location(client,
                                                    operation_name,
                                                    parameters)
        if not redirect_url or not is_absolute_url(redirect_url):
            raise RedirectExtensionError(
                err_msg='Missing or incorrect redirect URL.')
        operation_model.http['requestUri'] = redirect_url
        parsed_globals.endpoint_url = redirect_url
        # The command processing is not finished, continue to run other CLI callers.
        return True

    def _find_redirect_location(self, client, operation_name, parameters):
        # Sends an API request to find the redirect location.
        # We cannot use auto-redirect because we have to do the auth-sign
        # for the redirected location.
        http, resp = client.make_api_call(operation_name, parameters,
                                          allow_redirects=False)
        if http.is_redirect:
            redirect_url = http.headers.get('Location', None)
        else:
            redirect_url = None
        LOG.debug('Redirect request status: %d, redirect to: %s' %
                  (http.status_code, redirect_url))
        return redirect_url



# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Modifications made by Cloudera are:
#     Copyright (c) 2016 Cloudera, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from cdpcli import UNSIGNED
import cdpcli.auth
from cdpcli.exceptions import UnknownSignatureVersionError


class RequestSigner(object):
    """
    An object to sign requests before they go out over the wire using
    one of the authentication mechanisms defined in ``auth.py``.
    """
    def __init__(self, signature_version, credentials):
        self._signature_version = signature_version
        self._credentials = credentials

    @property
    def signature_version(self):
        return self._signature_version

    def sign(self, request):
        """
        Sign a request before it goes out over the wire.
        """
        if self._signature_version != UNSIGNED:
            signer = self.get_auth_instance(self._signature_version)
            signer.add_auth(request)

    def get_auth_instance(self, signature_version, **kwargs):
        """
        Get an auth instance which can be used to sign a request
        using the given signature version.
        """
        cls = cdpcli.auth.AUTH_TYPE_MAPS.get(signature_version)
        if cls is None:
            raise UnknownSignatureVersionError(
                signature_version=signature_version)
        # If there's no credentials provided (i.e credentials is None),
        # then we'll pass a value of "None" over to the auth classes,
        # which already handle the cases where no credentials have
        # been provided (by failing ...).
        frozen_credentials = self._credentials.get_frozen_credentials()
        kwargs['credentials'] = frozen_credentials
        auth = cls(**kwargs)
        return auth



# Copyright 2012-2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Modifications made by Cloudera are:
#     Copyright (c) 2016 Cloudera, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import copy
import os
import uuid

from cdpcli import credentials, DEFAULT_PROFILE_NAME
from cdpcli import xform_name
from cdpcli.auth import AccessTokenAuth
from cdpcli.auth import ECDSAv1Auth
from cdpcli.auth import Ed25519v1Auth
from cdpcli.auth import RSAv1Auth
from cdpcli.cdprequest import prepare_request_dict
from cdpcli.configloader import load_config, raw_config_parse
from cdpcli.endpoint import DEFAULT_TIMEOUT
from cdpcli.exceptions import ClientError, ConfigNotFound, ProfileNotFound
from cdpcli.exceptions import OperationNotPageableError
from cdpcli.model import ServiceModel
from cdpcli.paginate import Paginator
from cdpcli.serialize import create_serializer
from cdpcli.signers import RequestSigner
from cdpcli.utils import get_service_module_name

ALTUS_REQUEST_ID_HEADER = 'x-altus-request-id'
CDP_REQUEST_ID_HEADER = 'x-cdp-request-id'


class ClientCreator(object):

    def __init__(self,
                 loader,
                 context,
                 endpoint_creator,
                 user_agent_header,
                 response_parser_factory,
                 retryhandler):
        self._loader = loader
        self._context = context
        self._endpoint_creator = endpoint_creator
        self._user_agent_header = user_agent_header
        self._response_parser_factory = response_parser_factory
        self._retryhandler = retryhandler

    @property
    def context(self):
        return self._context

    def create_client(self,
                      service_name,
                      explicit_endpoint_url,
                      region,
                      tls_verification,
                      credentials,
                      client_config=None):
        service_model = self._load_service_model(service_name)
        cls = self._create_client_class(service_name, service_model)
        client_args = self._get_client_args(service_model,
                                            explicit_endpoint_url,
                                            region,
                                            tls_verification,
                                            credentials,
                                            client_config)
        return cls(**client_args)

    def _load_service_model(self, service_name):
        service_data = self._loader.load_service_data(service_name)
        return ServiceModel(service_data, service_name=service_name)

    def _create_client_class(self, service_name, service_model):
        class_attributes = self._create_methods(service_model)
        py_name_to_operation_name = self._create_name_mapping(service_model)
        class_attributes['_PY_TO_OP_NAME'] = py_name_to_operation_name
        bases = [BaseClient]
        class_name = get_service_module_name(service_model)
        cls = type(str(class_name), tuple(bases), class_attributes)
        return cls

    def _get_client_args(self,
                         service_model,
                         explicit_endpoint_url,
                         region,
                         tls_verification,
                         credentials,
                         client_config):
        """Get keyword arguments for constructing a client object.
        """
        serializer = create_serializer()
        if client_config is not None:
            connect_timeout = client_config.connect_timeout
            if connect_timeout <= 0:
                connect_timeout = None
            read_timeout = client_config.read_timeout
            if read_timeout <= 0:
                read_timeout = None
        else:
            connect_timeout = DEFAULT_TIMEOUT
            read_timeout = DEFAULT_TIMEOUT
        endpoint = self._endpoint_creator.create_endpoint(
            service_model,
            explicit_endpoint_url,
            self._context.get_scoped_config(),
            region,
            self._response_parser_factory,
            tls_verification,
            (connect_timeout, read_timeout),
            self._retryhandler)
        if AccessTokenAuth.is_access_token(credentials.access_token):
            auth_method = AccessTokenAuth.AUTH_METHOD_NAME
        elif Ed25519v1Auth.detect_private_key(credentials.private_key):
            auth_method = Ed25519v1Auth.AUTH_METHOD_NAME
        elif ECDSAv1Auth.detect_private_key(credentials.private_key):
            auth_method = ECDSAv1Auth.AUTH_METHOD_NAME
        else:
            auth_method = RSAv1Auth.AUTH_METHOD_NAME
        additional_headers = dict()
        request_headers_string = self._context.get_scoped_config().get(
            'request_headers', '')
        for header_string in request_headers_string.split(','):
            header_string = header_string.strip()
            if header_string == '':
                continue
            name, value = header_string.split('=', 1)
            if value == 'generate::uuid':
                value = str(uuid.uuid4())
            additional_headers[name] = value
        signer = RequestSigner(auth_method, credentials)
        return {
            'serializer': serializer,
            'service_model': service_model,
            'endpoint': endpoint,
            'loader': self._loader,
            'user_agent_header': self._user_agent_header,
            'additional_headers': additional_headers,
            'request_signer': signer
        }

    def _create_methods(self, service_model):
        op_dict = {}
        for operation_name in service_model.operation_names:
            py_operation_name = xform_name(operation_name)
            op_dict[py_operation_name] = self._create_api_method(
                py_operation_name, operation_name, service_model)
        return op_dict

    def _create_name_mapping(self, service_model):
        mapping = {}
        for operation_name in service_model.operation_names:
            py_operation_name = xform_name(operation_name)
            mapping[py_operation_name] = operation_name
        return mapping

    def _create_api_method(self, py_operation_name, operation_name, service_model):
        def _api_call(self, *args, **kwargs):
            if args:
                raise TypeError(
                    "%s() only accepts keyword arguments." % py_operation_name)
            http, parsed_response = self.make_api_call(operation_name, kwargs)
            return parsed_response

        _api_call.__name__ = str(py_operation_name)
        return _api_call


class BaseClient(object):

    # This is actually reassigned with the py->op_name mapping when the client
    # creator creates the subclass.  This value is used because calls such as
    # client.get_paginator('list_objects') use the snake_case name, but we need
    # to know the ListObjects form. The xform_name() function does the forward
    # mapping for ListObjects->list_objects conversion, but this holds the
    # reverse mapping.
    _PY_TO_OP_NAME = {}

    def __init__(self,
                 serializer,
                 endpoint,
                 service_model,
                 loader,
                 user_agent_header,
                 additional_headers,
                 request_signer):
        self._serializer = serializer
        self._endpoint = endpoint
        self._loader = loader
        self._user_agent_header = user_agent_header
        self._additional_headers = additional_headers
        self._request_signer = request_signer
        self.meta = ClientMeta(service_model, endpoint.host, self._PY_TO_OP_NAME)

    def can_paginate(self, operation_name):
        actual_operation_name = self._PY_TO_OP_NAME[operation_name]
        return self._service_model.operation_model(actual_operation_name).can_paginate

    def get_paginator(self, operation_name):
        if not self.can_paginate(operation_name):
            raise OperationNotPageableError(operation_name=operation_name)
        actual_operation_name = self._PY_TO_OP_NAME[operation_name]
        operation_model = self._service_model.operation_model(actual_operation_name)
        method = getattr(self, operation_name)
        return Paginator(method, operation_model)

    @property
    def _service_model(self):
        return self.meta.service_model

    def make_api_call(self, operation_name, api_params,
                      allow_redirects=True):
        operation_model = self._service_model.operation_model(operation_name)
        request_dict = self._serializer.serialize_to_request(api_params, operation_model)
        return self._make_request(operation_name, operation_model, request_dict,
                                  allow_redirects)

    def make_request(self, operation_name,
                     method, url_path, headers, body,
                     allow_redirects=True):
        operation_model = self._service_model.operation_model(operation_name)
        request_dict = {
            'url_path': url_path,
            'method': method,
            'headers': headers,
            'body': body
        }
        return self._make_request(operation_name, operation_model, request_dict,
                                  allow_redirects)

    def raise_error(self, operation_name, http, parsed_response):
        request_id = http.headers.get(ALTUS_REQUEST_ID_HEADER) \
            if ALTUS_REQUEST_ID_HEADER in http.headers \
            else http.headers.get(CDP_REQUEST_ID_HEADER, 'Unknown')
        raise ClientError(
            parsed_response,
            operation_name,
            self._service_model.service_name,
            http.status_code,
            request_id)

    def _make_request(self, operation_name, operation_model, request_dict,
                      allow_redirects):
        prepare_request_dict(request_dict,
                             endpoint_url=self._endpoint.host,
                             user_agent_header=self._user_agent_header,
                             additional_headers=self._additional_headers)
        http, parsed_response = self._endpoint.make_request(
            operation_model, request_dict, self._request_signer,
            allow_redirects=allow_redirects)
        if allow_redirects and http.status_code >= 300:
            self.raise_error(operation_name, http, parsed_response)
        elif not allow_redirects and http.status_code >= 400:
            self.raise_error(operation_name, http, parsed_response)
        else:
            return http, parsed_response


class ClientMeta(object):

    def __init__(self, service_model, endpoint_url, method_to_api_mapping):
        self._service_model = service_model
        self._endpoint_url = endpoint_url
        self._method_to_api_mapping = method_to_api_mapping

    @property
    def service_model(self):
        return self._service_model

    @property
    def endpoint_url(self):
        return self._endpoint_url

    @property
    def method_to_api_mapping(self):
        return self._method_to_api_mapping


class Context(object):
    """
    The Context object exposes configuration information and credentials into a
    single, easy-to-use object.

    """

    #: A default dictionary that maps the logical names for context variables
    #: to the specific environment variables and configuration file names
    #: that contain the values for these variables.
    #: When creating a new Context object, you can pass in your own dictionary
    #: to remap the logical names or to add new logical names.  You can then
    #: get the current value for these variables by using the
    #: ``get_config_variable`` method of the :class:`Context` #: class.
    #: These form the keys of the dictionary.  The values in the dictionary
    #: are tuples of (<config_name>, <environment variable>, <default value>,
    #: <conversion func>).
    #: The conversion func is a function that takes the configuration value
    #: as an argument and returns the converted value.  If this value is
    #: None, then the configuration value is returned unmodified.  This
    #: conversion function can be used to type convert config values to
    #: values other than the default values of strings.
    #: The ``profile`` and ``config_file`` variables should always have a
    #: None value for the first entry in the tuple because it doesn't make
    #: sense to look inside the config file for the location of the config
    #: file or for the default profile to use.
    #: The ``config_name`` is the name to look for in the configuration file,
    #: the ``env var`` is the OS environment variable (``os.environ``) to
    #: use, and ``default_value`` is the value to use if no value is otherwise
    #: found.
    CONTEXT_VARIABLES = {
        # logical:  config_file, env_var, default_value, conversion_func
        'profile': (None, ['CDP_DEFAULT_PROFILE', 'CDP_PROFILE'], None, None),
        'config_file': (None, 'CDP_CONFIG_FILE', '~/.cdp/config', None),
        'ca_bundle': ('ca_bundle', 'CDP_CA_BUNDLE', None, None),
        'api_versions': ('api_versions', None, {}, None),
        # This is the legacy json configuration file
        'auth_config': (None, None, None, None),
        # This is the shared credentials file.
        'credentials_file': (None, 'CDP_SHARED_CREDENTIALS_FILE',
                             '~/.cdp/credentials', None),
        'cdp_region': ('cdp_region', None, None, None),
        'cdp_endpoint_url': ('cdp_endpoint_url', None, None, None),
        'endpoint_url': ('endpoint_url', None, None, None),
        'form_factor': ('form_factor', None, None, None),
    }

    def __init__(self, profile=None):
        self.context_var_map = copy.copy(self.CONTEXT_VARIABLES)
        # The _profile attribute is just used to cache the value
        # of the current profile to avoid going through the normal
        # config lookup process each access time.
        self._profile = None
        self._config = None
        self._profile_map = None
        # This is a dict that stores per context specific config variable
        # overrides via set_config_variable().
        self._context_instance_vars = {}
        if profile is not None:
            self._context_instance_vars['profile'] = profile
        self._client_config = None

    @property
    def profile(self):
        """
        Returns the profile if one was set. Can be None.
        """
        if self._profile is None:
            profile = self.get_config_variable('profile')
            self._profile = profile
        return self._profile

    @property
    def effective_profile(self):
        """
        Returns the effective profile, cannot be None. If no profile was
        set the default profile is returned.
        """
        profile_name = self.profile
        if profile_name is None:
            profile_name = DEFAULT_PROFILE_NAME
        return profile_name

    @property
    def full_config(self):
        """Return the parsed config file.
        The ``get_config`` method returns the config associated with the
        specified profile.  This property returns the contents of the
        **entire** config file.
        """
        if self._config is None:
            try:
                config_file = self.get_config_variable('config_file')
                self._config = load_config(config_file)
            except ConfigNotFound:
                self._config = {'profiles': {}}
            try:
                # Now we need to inject the profiles from the
                # credentials file.  We don't actually need the values
                # in the creds file, only the profile names so that we
                # can validate the user is not referring to a nonexistent
                # profile.
                cred_file = self.get_config_variable('credentials_file')
                cred_profiles = raw_config_parse(cred_file)
                for profile in cred_profiles:
                    cred_vars = cred_profiles[profile]
                    if profile not in self._config['profiles']:
                        self._config['profiles'][profile] = cred_vars
                    else:
                        self._config['profiles'][profile].update(cred_vars)
            except ConfigNotFound:
                pass
        return self._config

    def _build_profile_map(self):
        # This will build the profile map if it has not been created,
        # otherwise it will return the cached value.  The profile map
        # is a list of profile names, to the config values for the profile.
        if self._profile_map is None:
            self._profile_map = self.full_config['profiles']
        return self._profile_map

    def get_config_variable(self, logical_name,
                            methods=('instance', 'env', 'config')):
        """
        Retrieve the value associated with the specified logical_name
        from the environment or the config file.  Values found in the
        environment variable take precedence of values found in the
        config file.  If no value can be found, a None will be returned.

        :type logical_name: str
        :param logical_name: The logical name of the context variable
            you want to retrieve.  This name will be mapped to the
            appropriate environment variable name for this context as
            well as the appropriate config file entry.

        :type methods: tuple
        :param methods: Defines which methods will be used to find
            the variable value.  By default, all available methods
            are tried but you can limit which methods are used
            by supplying a different value to this parameter.
            Valid choices are: instance|env|config

        :returns: value of variable or None if not defined.

        """
        # Handle all the short circuit special cases first.
        if logical_name not in self.context_var_map:
            return
        # Do the actual lookups.  We need to handle
        # 'instance', 'env', and 'config' locations, in that order.
        value = None
        var_config = self.context_var_map[logical_name]
        if self._found_in_instance_vars(methods, logical_name):
            return self._context_instance_vars[logical_name]
        elif self._found_in_env(methods, var_config):
            value = self._retrieve_from_env(var_config[1], os.environ)
        elif self._found_in_config_file(methods, var_config):
            value = self.get_scoped_config()[var_config[0]]
        if value is None:
            value = var_config[2]
        if var_config[3] is not None:
            value = var_config[3](value)
        return value

    def _found_in_instance_vars(self, methods, logical_name):
        if 'instance' in methods:
            return logical_name in self._context_instance_vars
        return False

    def _found_in_env(self, methods, var_config):
        return (
            'env' in methods and
            var_config[1] is not None and
            self._retrieve_from_env(var_config[1], os.environ) is not None)

    def _found_in_config_file(self, methods, var_config):
        if 'config' in methods and var_config[0] is not None:
            return var_config[0] in self.get_scoped_config()
        return False

    def _retrieve_from_env(self, names, environ):
        # We need to handle the case where names is either
        # a single value or a list of variables.
        if not isinstance(names, list):
            names = [names]
        for name in names:
            if name in environ:
                return environ[name]
        return None

    def get_scoped_config(self):
        """
        Returns the config values from the config file scoped to the current
        profile.

        The configuration data is loaded **only** from the config file.
        It does not resolve variables based on different locations
        (e.g. first from the context instance, then from environment
        variables, then from the config file).  If you want this lookup
        behavior, use the ``get_config_variable`` method instead.

        Note that this configuration is specific to a single profile (the
        ``profile`` session variable).

        If the ``profile`` session variable is set and the profile does
        not exist in the config file, a ``ProfileNotFound`` exception
        will be raised.
        """
        profile_name = self.get_config_variable('profile')
        profile_map = self._build_profile_map()
        # If a profile is not explicitly set return the default
        # profile config or an empty config dict if we don't have
        # a default profile.
        if profile_name is None:
            return profile_map.get('default', {})
        elif profile_name not in profile_map:
            # Otherwise if they specified a profile, it has to
            # exist (even if it's the default profile) otherwise
            # we complain.
            raise ProfileNotFound(profile=profile_name)
        else:
            return profile_map[profile_name]

    def set_config_variable(self, logical_name, value):
        """Set a configuration variable to a specific value.

        By using this method, you can override the normal lookup
        process used in ``get_config_variable`` by explicitly setting
        a value.  Subsequent calls to ``get_config_variable`` will
        use the ``value``.  This gives you per-session specific
        configuration values.
        ::
            >>> # Assume logical name 'foo' maps to env var 'FOO'
            >>> os.environ['FOO'] = 'myvalue'
            >>> s.get_config_variable('foo')
            'myvalue'
            >>> s.set_config_variable('foo', 'othervalue')
            >>> s.get_config_variable('foo')
            'othervalue'
        """
        self._context_instance_vars[logical_name] = value

    def get_credentials(self, parsed_globals=None):
        """
        Return the :class:`botocore.credential.Credential` object
        associated with this session.
        """
        resolver = credentials.create_credential_resolver(self, parsed_globals)
        return resolver.load_credentials()



# Copyright 2012-2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Modifications made by Cloudera are:
#     Copyright (c) 2016 Cloudera, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from copy import copy
import logging

from cdpcli.exceptions import PaginationError
from cdpcli.model import OperationModel

LOG = logging.getLogger('cdpcli.paginate')


class PageIterator(object):
    def __init__(self,
                 method,
                 operation_model,
                 max_items,
                 starting_token,
                 page_size,
                 op_kwargs):
        self._op_kwargs = op_kwargs
        # copy only those op model values that we do use
        self._method = method
        self._input_token_key = operation_model.paging_input_token
        self._output_token_key = operation_model.paging_output_token
        self._result_key = operation_model.paging_result
        self._page_size_key = operation_model.paging_page_size
        self._page_size_maximum = \
            operation_model.input_shape.members[self._page_size_key].maximum
        self._max_items = max_items if max_items is not None else \
            operation_model.paging_default_max_items
        self._page_size = page_size if page_size is not None \
            else self._max_items
        self._previous_next_token = starting_token

    def __iter__(self):
        previous_next_token = self._previous_next_token
        if previous_next_token is not None:
            self._op_kwargs[self._input_token_key] = previous_next_token
        items_returned = 0

        while True:
            # use explicit page sizes when --max-items is specified
            # the size of the last page we fetch must be such that we get
            # exactly max_items number of items
            self._op_kwargs[self._page_size_key] = min(
                self._page_size,
                self._page_size_maximum,
                self._max_items - items_returned
            )

            response = self._method(**self._op_kwargs)
            # yield a page of results to accumulator
            yield response

            # grab the result so we can update the returned item counter
            result = response.get(self._result_key, [])
            # if returned object is not a list, then we don't count
            # Note: this code and tests are likely dead, see THUN-637
            items_returned += len(result) if isinstance(result, list) else 0
            # grab continuation token from response presence of which means more
            # pages
            next_token = response.get(self._output_token_key, None)

            # validate runtime state, raise errors instead of producing possibly
            # incorrect results or hammering the service in unexpected ways
            if previous_next_token is not None and \
                    previous_next_token == next_token:
                msg = "The same next token was received twice: %s"
                raise PaginationError(message=msg % next_token)
            if items_returned > self._max_items:
                msg = "Received more items than expected. Expected: %d; " \
                      "Returned: %d" % (self._max_items, items_returned)
                raise PaginationError(message=msg)

            # exit if we reached the last page
            if next_token is None:
                break
            # exit if we returned the request max number of items
            if self._max_items == items_returned:
                # We should do better than this and return the CLI next token.
                # but that will come in a follow up change.
                LOG.warn("Max items received. Refine your filter or use "
                         "advance pagination.")
                break

            # advance to the next page
            self._op_kwargs[self._input_token_key] = \
                previous_next_token = next_token

    # builds a full operation result object by iterating over pages which are
    # fetched using multiple requests to service
    def build_full_result(self):
        result = {}
        # iterate over service responses, one response per page of results
        for response in self:
            response_value = response.get(self._result_key, None)
            if response_value is None:
                continue
            existing_value = result.get(self._result_key, None)
            if existing_value is None:
                result[self._result_key] = copy(response_value)
            elif isinstance(response_value, list):
                existing_value.extend(response_value)
            else:
                result[self._result_key] = existing_value + response_value

        # expose continuation token in the response, this should only happen if
        # not all items were fetched due to logic above, such as when
        # --max-items is used
        next_token = response.get(self._output_token_key, None)
        if next_token:
            result['nextToken'] = next_token

        return result


class Paginator(object):
    def __init__(self,
                 method,
                 operation_model):
        def assert_check(condition, message):
            if not condition:
                raise AssertionError(message)
        assert_check(callable(method), "method is callable")
        assert_check(isinstance(operation_model, OperationModel),
                     "operation_model has type OperationModel")
        self._method = method
        self._operation_model = operation_model

    def paginate(self, **kwargs):
        paging_params = self._extract_paging_params(kwargs)
        return PageIterator(self._method,
                            self._operation_model,
                            paging_params['MaxItems'],
                            paging_params['StartingToken'],
                            paging_params['PageSize'],
                            kwargs)

    def _extract_paging_params(self, kwargs):
        pagination_config = kwargs.pop('PaginationConfig', {})
        max_items = pagination_config.get('MaxItems', None)
        if max_items is not None:
            max_items = int(max_items)
        page_size = pagination_config.get('PageSize', None)
        if page_size is not None:
            page_size = int(page_size)
        return {
            'MaxItems': max_items,
            'StartingToken': pagination_config.get('StartingToken', None),
            'PageSize': page_size,
        }



# Copyright 2012-2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Modifications made by Cloudera are:
#     Copyright (c) 2016 Cloudera, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import argparse
from difflib import get_close_matches
import logging
import os
import sys

from cdpcli import ARGPARSE_DASH_ENCODING
from cdpcli.compat import six


LOG = logging.getLogger('cdpcli.argparser')


class CLIArgParser(argparse.ArgumentParser):
    Formatter = argparse.RawTextHelpFormatter

    # When displaying invalid choice error messages, this controls how many
    # options to show per line.
    ChoicesPerLine = 2

    def _check_value(self, action, value):
        """
        It's probably not a great idea to override a "hidden" method but the default
        behavior is pretty ugly and there doesn't seem to be any other way to change
        it.
        """
        # converted value must be one of the choices (if specified)
        if action.choices is not None and value not in action.choices:
            choice_max_len = len(max(action.choices, key=len))
            column_width = max(choice_max_len, 40)
            msg = ['Invalid choice, valid choices are:\n']
            for i in range(len(action.choices))[::self.ChoicesPerLine]:
                current = []
                for choice in action.choices[i:i + self.ChoicesPerLine]:
                    current.append(('%-' + str(column_width) + 's') % choice)
                msg.append(' | '.join(current))
            possible = get_close_matches(value, action.choices, cutoff=0.8)
            if possible:
                extra = ['\n\nInvalid choice: %r, maybe you meant:\n' % value]
                for word in possible:
                    extra.append('  * %s' % word)
                msg.extend(extra)
            raise argparse.ArgumentError(action, '\n'.join(msg))

    def parse_known_args(self, args, namespace=None):
        parsed, remaining = super(CLIArgParser, self).parse_known_args(args, namespace)
        terminal_encoding = getattr(sys.stdin, 'encoding', 'utf-8')
        if terminal_encoding is None:
            # In some cases, sys.stdin won't have an encoding set,
            # (e.g if it's set to a StringIO).  In this case we just
            # default to utf-8.
            terminal_encoding = 'utf-8'
        for arg, value in vars(parsed).items():
            if isinstance(value, six.binary_type):
                setattr(parsed,
                        arg,
                        self._decode_dash_if_necessary(
                            value.decode(terminal_encoding)))
            elif isinstance(value, six.string_types):
                setattr(parsed, arg, self._decode_dash_if_necessary(value))
            elif isinstance(value, list):
                encoded = []
                for v in value:
                    if isinstance(v, six.binary_type):
                        encoded.append(self._decode_dash_if_necessary(
                            v.decode(terminal_encoding)))
                    else:
                        if isinstance(v, six.string_types):
                            v = self._decode_dash_if_necessary(v)
                        encoded.append(v)
                setattr(parsed, arg, encoded)
        LOG.debug("parse_known_args called with args: %s. Parsed args: %s, "
                  "remaining: %s" % (args, parsed, remaining))
        return parsed, remaining

    def _decode_dash_if_necessary(self, arg_value):
        # Python argparse has a bug in which '-' are not parsed correctly if
        # they appear # as values for other arguments, see:
        # http://bugs.python.org/issue9334 for more details. This defines
        # special encoding for dash that we will "decode" and replace with a
        # dash. The reason we are using \\ is that there is a non zero chance
        # that customers can discover this themselves.
        if arg_value.startswith(ARGPARSE_DASH_ENCODING):
            return arg_value.replace(ARGPARSE_DASH_ENCODING, '-', 1)
        return arg_value


class MainArgParser(CLIArgParser):
    Formatter = argparse.RawTextHelpFormatter

    def __init__(self,
                 command_table,
                 version_string,
                 description,
                 argument_table):
        super(MainArgParser, self).__init__(
            formatter_class=self.Formatter,
            add_help=False,
            conflict_handler='resolve',
            description=description,
            allow_abbrev=False)
        self._build(command_table, version_string, argument_table)
        self._command_table = command_table

    def _create_choice_help(self, choices):
        help_str = ''
        for choice in sorted(choices):
            help_str += '* %s\n' % choice
        return help_str

    def _build(self, command_table, version_string, argument_table):
        for argument_name in argument_table:
            argument = argument_table[argument_name]
            argument.add_to_parser(self)
        self.add_argument('--version',
                          action="version",
                          version=version_string,
                          help='Display the version of this tool')
        self.add_argument('command', choices=list(command_table.keys()))

    def parse_known_args(self, args, namespace=None):
        """
        Parses normally, but also adds the command_table to the parsed
        arguments, so that commands like refdoc may access it. This is a hack.
        """
        parsed, remaining = super(MainArgParser, self).parse_known_args(args, namespace)
        setattr(parsed, 'command_table', self._command_table)
        return parsed, remaining


class ArgTableArgParser(CLIArgParser):

    def __init__(self, argument_table, service_name=None, operation_name=None,
                 command_table=None):
        # command_table is an optional subcommand_table.  If it's passed
        # in, then we'll update the argparse to parse a 'subcommand' argument
        # and populate the choices field with the command table keys.
        if service_name and operation_name:
            progString = "{} {} {}".format(os.path.basename(sys.argv[0]), service_name,
                                           operation_name)
        else:
            progString = os.path.basename(sys.argv[0])
        super(ArgTableArgParser, self).__init__(
            prog=progString,
            formatter_class=self.Formatter,
            add_help=False,
            conflict_handler='resolve')
        if command_table is None:
            command_table = {}
        self._build(argument_table, command_table)

    def _build(self, argument_table, command_table):
        for arg_name in argument_table:
            argument = argument_table[arg_name]
            argument.add_to_parser(self)
        if command_table:
            self.add_argument('subcommand', choices=list(command_table.keys()),
                              nargs='?')

    def parse_known_args(self, args, namespace=None):
        if len(args) == 1 and (args[0] == 'help' or args[0] == '--help'):
            namespace = argparse.Namespace()
            namespace.help = 'help'
            return namespace, []
        else:
            return super(ArgTableArgParser, self).parse_known_args(
                args, namespace)


class ServiceArgParser(CLIArgParser):

    def __init__(self, operations_table, service_name):
        super(ServiceArgParser, self).__init__(
            prog="{} {}".format(os.path.basename(sys.argv[0]), service_name),
            formatter_class=argparse.RawTextHelpFormatter,
            add_help=False,
            conflict_handler='resolve')
        self._build(operations_table)
        self._service_name = service_name

    def _build(self, operations_table):
        self.add_argument('operation', choices=list(operations_table.keys()))

    def parse_known_args(self, args, namespace=None):
        if len(args) == 0 or (len(args) == 1 and args[0] == '--help'):
            args = ['help']
        return super(ServiceArgParser, self).parse_known_args(
            args, namespace)
